@@ -75,8 +75,8 @@ _("usage: %s [options] <file | -- [op [op] ..]>\n"
 "\n"
 "All parameters following -- are considered ops to be chained together\n"
 "into a small composition instead of using an xml file, this allows for\n"
-"easy testing of filters. Be aware that the default value will be used\n"
-"for all properties.\n")
+"easy testing of filters. After chaining a new op in properties can be set\n"
+"with property=value pairs as subsequent arguments.\n")
 , application_name);
     exit (0);
 }
@@ -325,7 +325,7 @@ parse_args (int    argc,
         }
 
         else if (match ("--")) {
-            o->rest = curr;
+            o->rest = curr + 1;
             break;
         }
 
