@@ -22,6 +22,10 @@
 #include "gegl-tile-handler.h"
 #include "gegl-tile-handler-cache.h"
 
+/***
+ * GeglTileHandlerZoom is a GeglTileHandler that handle the mipmapping process.
+ */
+
 G_BEGIN_DECLS
 
 #define GEGL_TYPE_TILE_HANDLER_ZOOM            (gegl_tile_handler_zoom_get_type ())
@@ -38,7 +42,6 @@ typedef struct _GeglTileHandlerZoomClass GeglTileHandlerZoomClass;
 struct _GeglTileHandlerZoom
 {
   GeglTileHandler       parent_instance;
-  GeglTileHandlerCache *cache;
   GeglTileBackend      *backend;
   GeglTileStorage      *tile_storage;
 };
@@ -48,10 +51,9 @@ struct _GeglTileHandlerZoomClass
   GeglTileHandlerClass  parent_class;
 };
 
-GType gegl_tile_handler_zoom_get_type (void) G_GNUC_CONST;
-GeglTileHandler *gegl_tile_handler_zoom_new (GeglTileBackend *backend,
-                                             GeglTileStorage *tile_storage,
-                                             GeglTileHandlerCache *cache);
+GType             gegl_tile_handler_zoom_get_type (void) G_GNUC_CONST;
+
+GeglTileHandler * gegl_tile_handler_zoom_new      (GeglTileBackend *backend);
 
 G_END_DECLS
 
