@@ -722,31 +722,20 @@ gegl_node_connect_from (GeglNode    *sink,
       real_source_pad_name = "output";
     }
 
-  {
-    GeglPad *pad;
-    GeglPad *other_pad = NULL;
-
-    pad = gegl_node_get_pad (real_sink, real_sink_pad_name);
-    if (pad)
-      other_pad = gegl_pad_get_connected_to (pad);
-    else
-      {
-        g_warning ("%s: Didn't find pad '%s' of '%s'",
-                   G_STRFUNC, real_sink_pad_name, gegl_node_get_debug_name (real_sink));
-      }
-
-    if (other_pad)
-      {
-        gegl_node_disconnect (real_sink, real_sink_pad_name);
-      }
-  }
   if (gegl_node_pads_exist (real_sink, real_sink_pad_name, real_source, real_source_pad_name))
     {
+      GeglPad        *other_pad;
       GeglPad        *sink_pad   = gegl_node_get_pad (real_sink, real_sink_pad_name);
       GeglPad        *source_pad = gegl_node_get_pad (real_source, real_source_pad_name);
-      GeglConnection *connection = gegl_pad_connect (sink_pad,
-                                                     source_pad);
+      GeglConnection *connection;
 
+      other_pad = gegl_pad_get_connected_to (sink_pad);
+      if (source_pad == other_pad)
+        return TRUE;
+
+      gegl_node_disconnect (real_sink, real_sink_pad_name);
+
+      connection = gegl_pad_connect (sink_pad, source_pad);
       gegl_connection_set_sink_node (connection, real_sink);
       gegl_connection_set_source_node (connection, real_source);
 
@@ -756,7 +745,7 @@ gegl_node_connect_from (GeglNode    *sink,
       g_signal_connect (G_OBJECT (real_source), "invalidated",
                         G_CALLBACK (gegl_node_source_invalidated), sink_pad);
 
-      gegl_node_property_changed (G_OBJECT (real_source->operation), NULL, real_source);
+      gegl_node_source_invalidated (real_source, &real_source->have_rect, sink_pad);
 
       return TRUE;
     }
@@ -1337,6 +1326,9 @@ gegl_node_set_valist (GeglNode    *self,
 
   g_object_freeze_notify (G_OBJECT (self));
 
+  if (self->operation)
+    g_object_freeze_notify (G_OBJECT (self->operation));
+
   property_name = first_property_name;
   while (property_name)
     {
@@ -1348,9 +1340,16 @@ gegl_node_set_valist (GeglNode    *self,
           op_class          = va_arg (var_args, gchar *);
           op_first_property = va_arg (var_args, gchar *);
 
+          if (self->operation)
+            g_object_thaw_notify (G_OBJECT (self->operation));
+
           /* pass the following properties as construction properties
            * to the operation */
           gegl_node_set_op_class (self, op_class, op_first_property, var_args);
+
+          if (self->operation)
+            g_object_freeze_notify (G_OBJECT (self->operation));
+
           break;
         }
       else
@@ -1400,6 +1399,9 @@ gegl_node_set_valist (GeglNode    *self,
       property_name = va_arg (var_args, gchar *);
     }
 
+  if (self->operation)
+    g_object_thaw_notify (G_OBJECT (self->operation));
+
   g_object_thaw_notify (G_OBJECT (self));
 }
 
@@ -2109,10 +2111,12 @@ gegl_node_get_pad_proxy (GeglNode    *node,
 
       if (!is_graph_input)
         {
-          g_signal_connect_swapped (G_OBJECT (nop), "computed",
-                                    G_CALLBACK (gegl_node_emit_computed), node);
-          g_signal_connect (G_OBJECT (nop), "invalidated",
-                            G_CALLBACK (graph_source_invalidated), node);
+          g_signal_connect_object (G_OBJECT (nop), "computed",
+                                   G_CALLBACK (gegl_node_emit_computed), node,
+                                   G_CONNECT_SWAPPED);
+          g_signal_connect_object (G_OBJECT (nop), "invalidated",
+                                   G_CALLBACK (graph_source_invalidated), node,
+                                   0);
         }
       return nop;
     }
