@@ -46,12 +46,12 @@ struct _ColorNameEntity
   const gfloat rgba_color[4];
 };
 
-static gboolean  parse_float_argument_list (GeglColor *color,
+static gboolean  parse_float_argument_list (float rgba_color[4],
                                             GScanner  *scanner,
                                             gint       num_arguments);
-static gboolean  parse_color_name (GeglColor   *color,
+static gboolean  parse_color_name (float rgba_color[4],
                                    const gchar *color_string);
-static gboolean  parse_hex (GeglColor   *color,
+static gboolean  parse_hex (float rgba_color[4],
                             const gchar *color_string);
 static void      set_property (GObject      *gobject,
                                guint         prop_id,
@@ -64,6 +64,8 @@ static void      get_property (GObject    *gobject,
 
 /* These color names are based on those defined in the HTML 4.01 standard. See
  * http://www.w3.org/TR/html4/types.html#h-6.5
+ *
+ * Note: these values are stored with gamma
  */
 static const ColorNameEntity color_names[] =
 {
@@ -78,7 +80,7 @@ static const ColorNameEntity color_names[] =
   { "green",   { 0.f,      0.50196f, 0.f,      1.f } },
   { "lime",    { 0.f,      1.f,      0.f,      1.f } },
   { "olive",   { 0.50196f, 0.50196f, 0.f,      1.f } },
-  { "yellow",  { 1.f,      1.f,      01.f,     1.f } },
+  { "yellow",  { 1.f,      1.f,      0.f,      1.f } },
   { "navy",    { 0.f,      0.f,      0.50196f, 1.f } },
   { "blue",    { 0.f,      0.f,      1.f,      1.f } },
   { "teal",    { 0.f,      0.50196f, 0.50196f, 1.f } },
@@ -122,7 +124,7 @@ gegl_color_class_init (GeglColorClass *klass)
 }
 
 static gboolean
-parse_float_argument_list (GeglColor *color,
+parse_float_argument_list (float rgba_color[4],
                            GScanner  *scanner,
                            gint       num_arguments)
 {
@@ -145,12 +147,12 @@ parse_float_argument_list (GeglColor *color,
         {
           case G_TOKEN_FLOAT:
             token_value = g_scanner_cur_value (scanner);
-            color->priv->rgba_color[i] = token_value.v_float;
+            rgba_color[i] = token_value.v_float;
             break;
 
           case G_TOKEN_INT:
             token_value = g_scanner_cur_value (scanner);
-            color->priv->rgba_color[i] = token_value.v_int64;
+            rgba_color[i] = token_value.v_int64;
             break;
 
           default:
@@ -179,7 +181,7 @@ parse_float_argument_list (GeglColor *color,
 }
 
 static gboolean
-parse_color_name (GeglColor   *color,
+parse_color_name (float rgba_color[4],
                   const gchar *color_string)
 {
   gsize i;
@@ -188,7 +190,7 @@ parse_color_name (GeglColor   *color,
     {
       if (g_ascii_strcasecmp (color_names[i].color_name, color_string) == 0)
         {
-          memcpy (color->priv->rgba_color, color_names[i].rgba_color, sizeof (color_names[i].rgba_color));
+          memcpy (rgba_color, color_names[i].rgba_color, sizeof (color_names[i].rgba_color));
           return TRUE;
         }
     }
@@ -197,7 +199,7 @@ parse_color_name (GeglColor   *color,
 }
 
 static gboolean
-parse_hex (GeglColor   *color,
+parse_hex (float rgba_color[4],
            const gchar *color_string)
 {
   gint              i;
@@ -212,7 +214,7 @@ parse_hex (GeglColor   *color,
           if (g_ascii_isxdigit (color_string[2 * i + 1]) &&
               g_ascii_isxdigit (color_string[2 * i + 2]))
             {
-              color->priv->rgba_color[i] = (g_ascii_xdigit_value (color_string[2 * i + 1]) << 4 |
+              rgba_color[i] = (g_ascii_xdigit_value (color_string[2 * i + 1]) << 4 |
                                      g_ascii_xdigit_value (color_string[2 * i + 2])) / 255.f;
             }
           else
@@ -232,7 +234,7 @@ parse_hex (GeglColor   *color,
         {
           if (g_ascii_isxdigit (color_string[i + 1]))
             {
-              color->priv->rgba_color[i] = (g_ascii_xdigit_value (color_string[i + 1]) << 4 |
+              rgba_color[i] = (g_ascii_xdigit_value (color_string[i + 1]) << 4 |
                                      g_ascii_xdigit_value (color_string[i + 1])) / 255.f;
             }
           else
@@ -259,15 +261,29 @@ gegl_color_float4 (GeglColor *self)
 #endif
 
 void
-gegl_color_get_rgba4f (GeglColor   *color,
-                       gfloat      *rgba)
+gegl_color_set_pixel (GeglColor   *color,
+                      const Babl  *format,
+                      const void  *pixel)
 {
-  gint i;
-
   g_return_if_fail (GEGL_IS_COLOR (color));
+  g_return_if_fail (format);
+  g_return_if_fail (pixel);
 
-  for (i=0; i< 4; i++)
-    rgba[i] = color->priv->rgba_color[i];
+  babl_process (babl_fish (format, babl_format ("RGBA float")),
+                pixel, color->priv->rgba_color, 1);
+}
+
+void
+gegl_color_get_pixel (GeglColor   *color,
+                      const Babl  *format,
+                      void        *pixel)
+{
+  g_return_if_fail (GEGL_IS_COLOR (color));
+  g_return_if_fail (format);
+  g_return_if_fail (pixel);
+
+  babl_process (babl_fish (babl_format ("RGBA float"), format),
+                color->priv->rgba_color, pixel, 1);
 }
 
 void
@@ -308,6 +324,8 @@ gegl_color_set_from_string (GeglColor   *self,
   GTokenType        token_type;
   GTokenValue       token_value;
   gboolean          color_parsing_successfull;
+  float rgba[4] = {0.0, 0.0, 0.0, 1.0};
+  const Babl *format = babl_format ("R'G'B'A float");
 
   scanner                               = g_scanner_new (NULL);
   scanner->config->cpair_comment_single = "";
@@ -319,28 +337,34 @@ gegl_color_set_from_string (GeglColor   *self,
   if (token_type == G_TOKEN_IDENTIFIER &&
       g_ascii_strcasecmp (token_value.v_identifier, "rgb") == 0)
     {
-      color_parsing_successfull = parse_float_argument_list (self, scanner, 3);
-      self->priv->rgba_color[3]       = 1.f;
+      color_parsing_successfull = parse_float_argument_list (rgba, scanner, 3);
+      format = babl_format ("RGBA float");
     }
   else if (token_type == G_TOKEN_IDENTIFIER &&
            g_ascii_strcasecmp (token_value.v_identifier, "rgba") == 0)
     {
-      color_parsing_successfull = parse_float_argument_list (self, scanner, 4);
+      rgba[3] = 1.0;
+      color_parsing_successfull = parse_float_argument_list (rgba, scanner, 4);
+      format = babl_format ("RGBA float");
     }
-  else if (token_type == '#') /* FIXME: Verify that this is a safe way to check for '#' */
+  else if (token_type == '#')
     {
-      color_parsing_successfull = parse_hex (self, color_string);
+      color_parsing_successfull = parse_hex (rgba, color_string);
     }
   else if (token_type == G_TOKEN_IDENTIFIER)
     {
-      color_parsing_successfull = parse_color_name (self, color_string);
+      color_parsing_successfull = parse_color_name (rgba, color_string);
     }
   else
     {
       color_parsing_successfull = FALSE;
     }
 
-  if (!color_parsing_successfull)
+  if (color_parsing_successfull)
+    {
+        gegl_color_set_pixel(self, format, rgba);
+    }
+  else 
     {
       memcpy (self->priv->rgba_color,
               parsing_error_color,
@@ -429,6 +453,20 @@ gegl_color_new (const gchar *string)
   return g_object_new (GEGL_TYPE_COLOR, NULL);
 }
 
+GeglColor *
+gegl_color_duplicate (GeglColor *color)
+{
+  GeglColor *new;
+
+  g_return_val_if_fail (GEGL_IS_COLOR (color), NULL);
+
+  new = g_object_new (GEGL_TYPE_COLOR, NULL);
+
+  memcpy (new->priv, color->priv, sizeof (GeglColorPrivate));
+
+  return new;
+}
+
 /* --------------------------------------------------------------------------
  * A GParamSpec class to describe behavior of GeglColor as an object property
  * follows.
@@ -453,8 +491,6 @@ gegl_param_color_init (GParamSpec *self)
   GEGL_PARAM_COLOR (self)->default_color = NULL;
 }
 
-GeglColor *
-gegl_param_spec_color_get_default (GParamSpec *self);
 GeglColor *
 gegl_param_spec_color_get_default (GParamSpec *self)
 {
@@ -476,16 +512,15 @@ gegl_param_color_finalize (GParamSpec *self)
   parent_class->finalize (self);
 }
 
-#if 0
 static void
-value_set_default (GParamSpec *param_spec,
-                   GValue     *value)
+gegl_param_color_set_default (GParamSpec *param_spec,
+                              GValue     *value)
 {
   GeglParamColor *gegl_color = GEGL_PARAM_COLOR (param_spec);
 
-  g_value_set_object (value, gegl_color->default_color);
+  if (gegl_color->default_color)
+    g_value_take_object (value, gegl_color_duplicate (gegl_color->default_color));
 }
-#endif
 
 GType
 gegl_param_color_get_type (void)
@@ -500,7 +535,7 @@ gegl_param_color_get_type (void)
         gegl_param_color_init,
         0,
         gegl_param_color_finalize,
-        NULL, /*value_set_default,*/
+        gegl_param_color_set_default,
         NULL,
         NULL
       };
@@ -524,10 +559,10 @@ gegl_param_spec_color (const gchar *name,
 
   param_color = g_param_spec_internal (GEGL_TYPE_PARAM_COLOR,
                                        name, nick, blurb, flags);
-/*
+
   param_color->default_color = default_color;
   if (default_color)
-    g_object_ref (default_color);*/
+    g_object_ref (default_color);
 
   return G_PARAM_SPEC (param_color);
 }
@@ -543,9 +578,10 @@ gegl_param_spec_color_from_string (const gchar *name,
 
   param_color = g_param_spec_internal (GEGL_TYPE_PARAM_COLOR,
                                        name, nick, blurb, flags);
-/*
+
   param_color->default_color = g_object_new (GEGL_TYPE_COLOR,
                                              "string", default_color_string,
-                                             NULL);*/
+                                             NULL);
+
   return G_PARAM_SPEC (param_color);
 }
