@@ -493,7 +493,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "  gegl_operation_class_set_keys (operation_class,                             \n"
 "  \"name\"       , \"gegl:bilateral-filter-fast\",                            \n"
-"  \"title\"      , \"Bilateral Box Filter\",                                  \n"
+"  \"title\"      , _(\"Bilateral Box Filter\"),                               \n"
 "  \"categories\" , \"enhance:noise-reduction\",                               \n"
 "  \"description\",                                                            \n"
 "           _(\"A fast approximation of bilateral filter, using a box-filter instead of a gaussian blur.\"),\n"
