@@ -19,29 +19,58 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_int_ui   (x,        _("Width"),  1, G_MAXINT, 32, 1, 128, 1.0,
-                  _("Horizontal width of cells pixels"))
-gegl_chant_int_ui   (y,        _("Height"), 1, G_MAXINT, 32, 1, 128, 1.0,
-                  _("Vertical width of cells in pixels"))
-gegl_chant_int_ui  (x_offset, _("X offset"), -G_MAXINT, G_MAXINT, 0, -64, 64, 1.0,
-                  _("Horizontal offset (from origin) for start of grid"))
-gegl_chant_int_ui  (y_offset, _("Y offset"), -G_MAXINT, G_MAXINT, 0, -64, 64, 1.0,
-                  _("Vertical offset (from origin) for start of grid"))
-gegl_chant_int_ui  (line_width, _("Line Width"),  0, G_MAXINT, 4, 0, 16, 1.0,
-                  _("Width of grid lines in pixels"))
-gegl_chant_int_ui  (line_height, _("Line Height"), 0, G_MAXINT, 4, 0, 16, 1.0,
-                  _("Height of grid lines in pixels"))
-gegl_chant_color (line_color,   _("Color"), "black",
-                  _("Color of the grid lines"))
+property_int    (x, _("Width"), 32)
+    description (_("Horizontal width of cells pixels"))
+    value_range (1, G_MAXINT)
+    ui_range    (1, 128)
+    ui_meta     ("unit", "pixel-distance")
+    ui_meta     ("axis", "x")
+
+property_int    (y, _("Height"), 32)
+    description (_("Vertical width of cells pixels"))
+    value_range (1, G_MAXINT)
+    ui_range    (1, 128)
+    ui_meta     ("unit", "pixel-distance")
+    ui_meta     ("axis", "y")
+
+property_int    (x_offset, _("Offset X"), 0)
+    description (_("Horizontal offset (from origin) for start of grid"))
+    ui_range    (-64, 64)
+    ui_meta     ("unit", "pixel-coordinate")
+    ui_meta     ("axis", "x")
+
+property_int    (y_offset, _("Offset Y"), 0)
+    description (_("Vertical offset (from origin) for start of grid"))
+    ui_range    (-64, 64)
+    ui_meta     ("unit", "pixel-coordinate")
+    ui_meta     ("axis", "y")
+
+property_int    (line_width, _("Line width"), 4)
+    description (_("Width of grid lines in pixels"))
+    value_range (0, G_MAXINT)
+    ui_range    (0, 16)
+    ui_meta     ("unit", "pixel-distance")
+    ui_meta     ("axis", "x")
+
+property_int    (line_height, _("Line height"), 4)
+    description (_("Height of grid lines in pixels"))
+    value_range (0, G_MAXINT)
+    ui_range    (0, 16)
+    ui_meta     ("unit", "pixel-distance")
+    ui_meta     ("axis", "y")
+
+property_color (line_color, _("Color"), "black")
+    description (_("Color of the grid lines"))
+    ui_meta     ("role",    "color-primary")
 
 #else
 
-#define GEGL_CHANT_TYPE_POINT_RENDER
-#define GEGL_CHANT_C_FILE       "grid.c"
+#define GEGL_OP_POINT_RENDER
+#define GEGL_OP_C_SOURCE grid.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 static void
 prepare (GeglOperation *operation)
@@ -62,7 +91,7 @@ process (GeglOperation       *operation,
          const GeglRectangle *roi,
          gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   gfloat     *out_pixel = out_buf;
   gfloat      line_color[4];
   gint        x = roi->x; /* initial x         */
@@ -109,9 +138,8 @@ process (GeglOperation       *operation,
   return  TRUE;
 }
 
-
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass            *operation_class;
   GeglOperationPointRenderClass *point_render_class;
@@ -124,9 +152,12 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->prepare = prepare;
 
   gegl_operation_class_set_keys (operation_class,
-    "name"       , "gegl:grid",
-    "categories" , "render",
-    "description", _("Grid renderer"),
+    "name",               "gegl:grid",
+    "title",              _("Grid"),
+    "position-dependent", "true",
+    "categories" ,        "render",
+    "description",        _("Grid renderer"),
+    "position-dependent", "true",
     NULL);
 }
 
