@@ -19,6 +19,11 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 #include <gegl-gio-private.h>
+#include <stdlib.h>
+
+#ifdef G_OS_WIN32
+#define realpath(a,b) _fullpath(b,a,_MAX_PATH)
+#endif
 
 #ifdef GEGL_PROPERTIES
 
@@ -83,7 +88,7 @@ do_setup (GeglOperation *operation, const gchar *new_path, const gchar *new_uri)
             extension = g_strdup(strrchr (new_uri, '.'));
 
         if (extension)
-            handler = gegl_extension_handler_get (extension);
+            handler = gegl_extension_handler_get_loader (extension);
         gegl_node_set (self->load, "operation", handler, NULL);
         gegl_node_set (self->load, "uri", new_uri, NULL);
 
@@ -93,10 +98,13 @@ do_setup (GeglOperation *operation, const gchar *new_path, const gchar *new_uri)
     {
       const gchar *extension = strrchr (new_path, '.');
       const gchar *handler   = NULL;
+      char   resolved_path[PATH_MAX];
 
-      if (!g_file_test (new_path, G_FILE_TEST_EXISTS))
+      realpath (new_path, resolved_path);
+
+      if (!g_file_test (resolved_path, G_FILE_TEST_EXISTS))
         {
-          gchar *name = g_filename_display_name (new_path);
+          gchar *name = g_filename_display_name (resolved_path);
           gchar *tmp  = g_strdup_printf ("File '%s' does not exist", name);
           g_free (name);
 
@@ -111,12 +119,12 @@ do_setup (GeglOperation *operation, const gchar *new_path, const gchar *new_uri)
       else
         {
           if (extension)
-            handler = gegl_extension_handler_get (extension);
+            handler = gegl_extension_handler_get_loader (extension);
           gegl_node_set (self->load,
                          "operation", handler,
                          NULL);
           gegl_node_set (self->load,
-                         "path", new_path,
+                         "path", resolved_path,
                          NULL);
         }
     }
@@ -199,20 +207,6 @@ my_set_property (GObject      *gobject,
   g_free (old_uri);
 }
 
-static void
-prepare (GeglOperation *operation)
-{
-  GeglOp  *self = GEGL_OP (operation);
-  GeglOperation *op;
-
-  /* forward the set BablFormat of the image loader on the meta-op itself,
-   * making potential cache buffers be created with the proper format, there
-   * might be cleaner ways of achieving this.
-   */
-  g_object_get (self->load, "gegl-operation", &op, NULL);
-  gegl_operation_set_format (operation, "output", gegl_operation_get_format (op, "output"));
-}
-
 static void
 gegl_op_class_init (GeglOpClass *klass)
 {
@@ -223,7 +217,6 @@ gegl_op_class_init (GeglOpClass *klass)
 
   operation_class->attach = attach;
   operation_class->detect = detect;
-  operation_class->prepare = prepare;
   operation_class->no_cache = TRUE;
 
   gegl_operation_class_set_keys (operation_class,
