@@ -20,6 +20,11 @@ static const char* op_c_source =
 "#include \"config.h\"                                                         \n"
 "#include <glib/gi18n-lib.h>                                                   \n"
 "#include <gegl-gio-private.h>                                                 \n"
+"#include <stdlib.h>                                                           \n"
+"                                                                              \n"
+"#ifdef G_OS_WIN32                                                             \n"
+"#define realpath(a,b) _fullpath(b,a,_MAX_PATH)                                \n"
+"#endif                                                                        \n"
 "                                                                              \n"
 "#ifdef GEGL_PROPERTIES                                                        \n"
 "                                                                              \n"
@@ -84,7 +89,7 @@ static const char* op_c_source =
 "            extension = g_strdup(strrchr (new_uri, '.'));                     \n"
 "                                                                              \n"
 "        if (extension)                                                        \n"
-"            handler = gegl_extension_handler_get (extension);                 \n"
+"            handler = gegl_extension_handler_get_loader (extension);          \n"
 "        gegl_node_set (self->load, \"operation\", handler, NULL);             \n"
 "        gegl_node_set (self->load, \"uri\", new_uri, NULL);                   \n"
 "                                                                              \n"
@@ -94,10 +99,13 @@ static const char* op_c_source =
 "    {                                                                         \n"
 "      const gchar *extension = strrchr (new_path, '.');                       \n"
 "      const gchar *handler   = NULL;                                          \n"
+"      char   resolved_path[PATH_MAX];                                         \n"
 "                                                                              \n"
-"      if (!g_file_test (new_path, G_FILE_TEST_EXISTS))                        \n"
+"      realpath (new_path, resolved_path);                                     \n"
+"                                                                              \n"
+"      if (!g_file_test (resolved_path, G_FILE_TEST_EXISTS))                   \n"
 "        {                                                                     \n"
-"          gchar *name = g_filename_display_name (new_path);                   \n"
+"          gchar *name = g_filename_display_name (resolved_path);              \n"
 "          gchar *tmp  = g_strdup_printf (\"File '%s' does not exist\", name); \n"
 "          g_free (name);                                                      \n"
 "                                                                              \n"
@@ -112,12 +120,12 @@ static const char* op_c_source =
 "      else                                                                    \n"
 "        {                                                                     \n"
 "          if (extension)                                                      \n"
-"            handler = gegl_extension_handler_get (extension);                 \n"
+"            handler = gegl_extension_handler_get_loader (extension);          \n"
 "          gegl_node_set (self->load,                                          \n"
 "                         \"operation\", handler,                              \n"
 "                         NULL);                                               \n"
 "          gegl_node_set (self->load,                                          \n"
-"                         \"path\", new_path,                                  \n"
+"                         \"path\", resolved_path,                             \n"
 "                         NULL);                                               \n"
 "        }                                                                     \n"
 "    }                                                                         \n"
@@ -201,20 +209,6 @@ static const char* op_c_source =
 "}                                                                             \n"
 "                                                                              \n"
 "static void                                                                   \n"
-"prepare (GeglOperation *operation)                                            \n"
-"{                                                                             \n"
-"  GeglOp  *self = GEGL_OP (operation);                                        \n"
-"  GeglOperation *op;                                                          \n"
-"                                                                              \n"
-"  /* forward the set BablFormat of the image loader on the meta-op itself,    \n"
-"   * making potential cache buffers be created with the proper format, there  \n"
-"   * might be cleaner ways of achieving this.                                 \n"
-"   */                                                                         \n"
-"  g_object_get (self->load, \"gegl-operation\", &op, NULL);                   \n"
-"  gegl_operation_set_format (operation, \"output\", gegl_operation_get_format (op, \"output\"));\n"
-"}                                                                             \n"
-"                                                                              \n"
-"static void                                                                   \n"
 "gegl_op_class_init (GeglOpClass *klass)                                       \n"
 "{                                                                             \n"
 "  GObjectClass       *object_class    = G_OBJECT_CLASS (klass);               \n"
@@ -224,7 +218,6 @@ static const char* op_c_source =
 "                                                                              \n"
 "  operation_class->attach = attach;                                           \n"
 "  operation_class->detect = detect;                                           \n"
-"  operation_class->prepare = prepare;                                         \n"
 "  operation_class->no_cache = TRUE;                                           \n"
 "                                                                              \n"
 "  gegl_operation_class_set_keys (operation_class,                             \n"
