@@ -61,7 +61,7 @@ static const char* op_c_source =
 "               GEGL_MAZE_ALGORITHM_DEPTH_FIRST)                               \n"
 "  description (_(\"Maze algorithm type\"))                                    \n"
 "                                                                              \n"
-"property_boolean (tilable, _(\"Tilable\"), FALSE)                             \n"
+"property_boolean (tileable, _(\"Tileable\"), FALSE)                           \n"
 "                                                                              \n"
 "property_seed (seed, _(\"Random seed\"), rand)                                \n"
 "                                                                              \n"
@@ -618,7 +618,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "      gr = g_rand_new ();                                                     \n"
 "                                                                              \n"
-"      if (o->tilable)                                                         \n"
+"      if (o->tileable)                                                        \n"
 "	{                                                                            \n"
 "	  /* Tileable mazes must be even. */                                         \n"
 "	  mw -= (mw & 1);                                                            \n"
@@ -637,7 +637,7 @@ static const char* op_c_source =
 "      switch (o->algorithm_type)                                              \n"
 "	{                                                                            \n"
 "	case GEGL_MAZE_ALGORITHM_DEPTH_FIRST:                                        \n"
-"	  if (o->tilable)                                                            \n"
+"	  if (o->tileable)                                                           \n"
 "	    {                                                                        \n"
 "	      depth_first_tileable (0, maz, mw, mh, o->seed);                        \n"
 "	    }                                                                        \n"
@@ -648,7 +648,7 @@ static const char* op_c_source =
 "	  break;                                                                     \n"
 "                                                                              \n"
 "	case GEGL_MAZE_ALGORITHM_PRIM:                                               \n"
-"	  if (o->tilable)                                                            \n"
+"	  if (o->tileable)                                                           \n"
 "	    {                                                                        \n"
 "	      prim_tileable (maz, mw, mh, o->seed);                                  \n"
 "	    }                                                                        \n"
@@ -697,7 +697,7 @@ static const char* op_c_source =
 "	    }                                                                        \n"
 "	}                                                                            \n"
 "                                                                              \n"
-"      if (! o->tilable)                                                       \n"
+"      if (! o->tileable)                                                      \n"
 "	{                                                                            \n"
 "          /* last row */                                                      \n"
 "	  for (tile.y = in_extent->height-tile.height, tile.x = offset_x;            \n"
