@@ -54,8 +54,8 @@ static const char* op_c_source =
 "    ui_range    (0, 15)                                                       \n"
 "    value_range (0, 15)                                                       \n"
 "                                                                              \n"
-"property_boolean (tilable, _(\"Tilable\"), FALSE)                             \n"
-"    description  (_(\"Create a tilable output\"))                             \n"
+"property_boolean (tileable, _(\"Tileable\"), FALSE)                           \n"
+"    description  (_(\"Create a tileable output\"))                            \n"
 "                                                                              \n"
 "property_boolean (turbulent, _(\"Turbulent\"), FALSE)                         \n"
 "    description  (_(\"Make a turbulent noise\"))                              \n"
@@ -121,7 +121,7 @@ static const char* op_c_source =
 "  gr = g_rand_new_with_seed (o->seed);                                        \n"
 "                                                                              \n"
 "  /*  Set scaling factors  */                                                 \n"
-"  if (o->tilable)                                                             \n"
+"  if (o->tileable)                                                            \n"
 "    {                                                                         \n"
 "      params->xsize = ceil (o->x_size);                                       \n"
 "      params->ysize = ceil (o->y_size);                                       \n"
@@ -202,7 +202,7 @@ static const char* op_c_source =
 "    {                                                                         \n"
 "      for (j = 0; j < 2; j++)                                                 \n"
 "        {                                                                     \n"
-"          if (o->tilable)                                                     \n"
+"          if (o->tileable)                                                    \n"
 "            n = p->perm_tab[(((a + i) % (p->xclip * s)) + p->perm_tab[((b + j) % (p->yclip * s)) % TABLE_SIZE]) % TABLE_SIZE];\n"
 "          else                                                                \n"
 "            n = p->perm_tab[(a + i + p->perm_tab[(b + j) % TABLE_SIZE]) % TABLE_SIZE];\n"
@@ -258,6 +258,21 @@ static const char* op_c_source =
 "  gegl_operation_set_format (operation, \"output\", format);                  \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static void                                                                   \n"
+"finalize (GObject *object)                                                    \n"
+"{                                                                             \n"
+"  GeglOperation *op = (void*) object;                                         \n"
+"  GeglProperties *o = GEGL_PROPERTIES (op);                                   \n"
+"                                                                              \n"
+"  if (o->user_data)                                                           \n"
+"    {                                                                         \n"
+"      g_slice_free (NsParamsType, o->user_data);                              \n"
+"      o->user_data = NULL;                                                    \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  G_OBJECT_CLASS (gegl_op_parent_class)->finalize (object);                   \n"
+"}                                                                             \n"
+"                                                                              \n"
 "static gboolean                                                               \n"
 "c_process (GeglOperation       *operation,                                    \n"
 "           void                *out_buf,                                      \n"
@@ -311,12 +326,16 @@ static const char* op_c_source =
 "static void                                                                   \n"
 "gegl_op_class_init (GeglOpClass *klass)                                       \n"
 "{                                                                             \n"
+"  GObjectClass             *object_class;                                     \n"
 "  GeglOperationClass       *operation_class;                                  \n"
 "  GeglOperationSourceClass *source_class;                                     \n"
 "                                                                              \n"
+"  object_class    = G_OBJECT_CLASS (klass);                                   \n"
 "  operation_class = GEGL_OPERATION_CLASS (klass);                             \n"
 "  source_class = GEGL_OPERATION_SOURCE_CLASS (klass);                         \n"
 "                                                                              \n"
+"  object_class->finalize = finalize;                                          \n"
+"                                                                              \n"
 "  source_class->process = process;                                            \n"
 "  operation_class->get_bounding_box = get_bounding_box;                       \n"
 "  operation_class->prepare = prepare;                                         \n"
