@@ -188,6 +188,53 @@ static const char* op_c_source =
 "  gegl_operation_set_format (operation, \"output\", format);                  \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static gboolean                                                               \n"
+"cl_process (GeglOperation       *operation,                                   \n"
+"            cl_mem               in_buf,                                      \n"
+"            cl_mem               out_buf,                                     \n"
+"            const size_t         n_pixels,                                    \n"
+"            const GeglRectangle *roi,                                         \n"
+"            gint                 level)                                       \n"
+"{                                                                             \n"
+"  GeglOperationClass *operation_class = GEGL_OPERATION_GET_CLASS (operation); \n"
+"  GeglClRunData *cl_data = operation_class->cl_data;                          \n"
+"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
+"  const size_t gbl_size[2] = {roi->width, roi->height};                       \n"
+"  const size_t gbl_off[2]  = {roi->x, roi->y};                                \n"
+"  cl_int cl_err = 0;                                                          \n"
+"                                                                              \n"
+"  cl_mem filter_pat = gegl_clCreateBuffer (gegl_cl_get_context (),            \n"
+"                                           CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,\n"
+"                                           pattern_width[o->pattern] *        \n"
+"                                           pattern_height[o->pattern] * sizeof(cl_int),\n"
+"                                           (void*)pattern[o->pattern],        \n"
+"                                           &cl_err);                          \n"
+"  CL_CHECK;                                                                   \n"
+"  cl_err = gegl_cl_set_kernel_args (cl_data->kernel[0],                       \n"
+"                                    sizeof(cl_mem), &in_buf,                  \n"
+"                                    sizeof(cl_mem), &out_buf,                 \n"
+"                                    sizeof(cl_mem), &filter_pat,              \n"
+"                                    sizeof(cl_int), &pattern_width[o->pattern],\n"
+"                                    sizeof(cl_int), &pattern_height[o->pattern],\n"
+"                                    sizeof(cl_int), &o->additive,             \n"
+"                                    sizeof(cl_int), &o->rotated,              \n"
+"                                    NULL);                                    \n"
+"  CL_CHECK;                                                                   \n"
+"  cl_err = gegl_clEnqueueNDRangeKernel (gegl_cl_get_command_queue (),         \n"
+"                                        cl_data->kernel[0], 2,                \n"
+"                                        gbl_off, gbl_size, NULL,              \n"
+"                                        0, NULL, NULL);                       \n"
+"  CL_CHECK;                                                                   \n"
+"  cl_err = gegl_clFinish (gegl_cl_get_command_queue ());                      \n"
+"  CL_CHECK;                                                                   \n"
+"  cl_err = gegl_clReleaseMemObject (filter_pat);                              \n"
+"  CL_CHECK;                                                                   \n"
+"  return FALSE;                                                               \n"
+"  error:                                                                      \n"
+"    if (filter_pat)                                                           \n"
+"      gegl_clReleaseMemObject (filter_pat);                                   \n"
+"    return TRUE;                                                              \n"
+"}                                                                             \n"
 "                                                                              \n"
 "static gboolean                                                               \n"
 "process (GeglOperation       *operation,                                      \n"
@@ -251,6 +298,8 @@ static const char* op_c_source =
 "  return TRUE;                                                                \n"
 "}                                                                             \n"
 "                                                                              \n"
+"#include \"opencl/video-degradation.cl.h\"                                    \n"
+"                                                                              \n"
 "static void                                                                   \n"
 "gegl_op_class_init (GeglOpClass *klass)                                       \n"
 "{                                                                             \n"
@@ -261,9 +310,9 @@ static const char* op_c_source =
 "  filter_class    = GEGL_OPERATION_POINT_FILTER_CLASS (klass);                \n"
 "                                                                              \n"
 "  operation_class->prepare = prepare;                                         \n"
-"  operation_class->opencl_support = FALSE;                                    \n"
 "                                                                              \n"
 "  filter_class->process    = process;                                         \n"
+"  filter_class->cl_process = cl_process;                                      \n"
 "                                                                              \n"
 "  gegl_operation_class_set_keys (operation_class,                             \n"
 "    \"name\",        \"gegl:video-degradation\",                              \n"
@@ -272,6 +321,7 @@ static const char* op_c_source =
 "    \"license\",     \"GPL3+\",                                               \n"
 "    \"description\", _(\"This function simulates the degradation of \"        \n"
 "                     \"being on an old low-dotpitch RGB video monitor.\"),    \n"
+"    \"cl-source\"  , video_degradation_cl_source,                             \n"
 "    NULL);                                                                    \n"
 "}                                                                             \n"
 "                                                                              \n"
