@@ -18,6 +18,11 @@
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
+#include <stdlib.h>
+
+#ifdef G_OS_WIN32
+#define realpath(a,b) _fullpath(b,a,_MAX_PATH)
+#endif
 
 #ifdef GEGL_PROPERTIES
 
@@ -53,8 +58,14 @@ property_audio_fragment (audio, _("audio"), 0)
 
 #include "gegl-op.h"
 #include <errno.h>
+#include <limits.h>
+#include <stdlib.h>
 
+#include <libavutil/avutil.h>
+#include <libavutil/imgutils.h>
 #include <libavformat/avformat.h>
+#include <libswscale/swscale.h>
+
 
 typedef struct
 {
@@ -81,6 +92,7 @@ typedef struct
   AVStream        *audio_stream;
   AVCodec         *video_codec;
   AVFrame         *lavc_frame;
+  AVFrame         *rgb_frame;
   glong            prevframe;      /* previously decoded frame number */
   gdouble          prevpts;        /* timestamp in seconds of last decoded frame */
 
@@ -116,6 +128,7 @@ clear_audio_track (GeglProperties *o)
       g_object_unref (p->audio_track->data);
       p->audio_track = g_list_remove (p->audio_track, p->audio_track->data);
     }
+  p->prevapts = 0.0;
 }
 
 static void
@@ -135,12 +148,15 @@ ff_cleanup (GeglProperties *o)
         avformat_close_input(&p->video_fcontext);
       if (p->audio_fcontext)
         avformat_close_input(&p->audio_fcontext);
+      if (p->rgb_frame)
+        av_free (p->rgb_frame);
       if (p->lavc_frame)
         av_free (p->lavc_frame);
 
       p->video_fcontext = NULL;
       p->audio_fcontext = NULL;
       p->lavc_frame = NULL;
+      p->rgb_frame = NULL;
       p->loadedfilename = NULL;
     }
 }
@@ -169,6 +185,7 @@ init (GeglProperties *o)
   p->loadedfilename = g_strdup ("");
 
   ff_cleanup (o);
+  av_log_set_level (AV_LOG_WARNING);
 }
 
 /* maintain list of audio samples */
@@ -180,22 +197,19 @@ decode_audio (GeglOperation *operation,
   GeglProperties *o = GEGL_PROPERTIES (operation);
   Priv       *p = (Priv*)o->user_data;
 
-  pts1 -= 15.0;
+  pts1 -= 2.0;
   if (pts1 < 0.0)pts1 = 0.0;
 
-  /* figure out which frame we should start decoding at */
-  //fprintf (stderr, "%f %f\n", p->prevapts, pts2);
-  if(0){
-  //int64_t seek_target = av_rescale_q ((pts1) * AV_TIME_BASE, AV_TIME_BASE_Q, p->audio_stream->time_base);
-  int64_t seek_target = pts1 * AV_TIME_BASE;
+  if(pts1 - 15.0 > p->prevapts){
+  int64_t seek_target = av_rescale_q (pts1 * AV_TIME_BASE, AV_TIME_BASE_Q, p->audio_stream->time_base);
+     clear_audio_track (o);
+     p->prevapts = 0.0;
 
-#if 0
-  clear_audio_track (o);
-#endif
-  p->prevapts = 0.0;
-
-    if (av_seek_frame (p->audio_fcontext, -1, seek_target, (AVSEEK_FLAG_BACKWARD)) < 0)
+     if (av_seek_frame (p->audio_fcontext, p->audio_stream->index, seek_target, (AVSEEK_FLAG_BACKWARD)) < 0)
       fprintf (stderr, "audio seek error!\n");
+     else
+      avcodec_flush_buffers (p->audio_stream->codec);
+
   }
 
   while (p->prevapts <= pts2)
@@ -205,7 +219,7 @@ decode_audio (GeglOperation *operation,
 
       if (av_read_frame (p->audio_fcontext, &pkt) < 0)
          {
-           av_free_packet (&pkt);
+           av_packet_unref (&pkt);
            return -1;
          }
       if (pkt.stream_index==p->audio_index && p->audio_stream)
@@ -233,8 +247,6 @@ decode_audio (GeglOperation *operation,
                GeglAudioFragment *af = gegl_audio_fragment_new (o->audio_sample_rate, channels,
                           AV_CH_LAYOUT_STEREO, samples_left);
 //);
-               //af->channels = MIN(p->audio_stream->codec->channels, GEGL_MAX_AUDIO_CHANNELS);
-
                switch (p->audio_stream->codec->sample_fmt)
                {
                  case AV_SAMPLE_FMT_FLT:
@@ -273,7 +285,8 @@ decode_audio (GeglOperation *operation,
                   g_warning ("undealt with sample format\n");
                 }
                 gegl_audio_fragment_set_sample_count (af, sample_count);
-                gegl_audio_fragment_set_pos (af, p->audio_pos);
+                gegl_audio_fragment_set_pos (af, 
+  (long int)av_rescale_q ((pkt.pts), p->audio_stream->time_base, AV_TIME_BASE_Q) * o->audio_sample_rate /AV_TIME_BASE);
 
                 p->audio_pos += sample_count;
                 p->audio_track = g_list_append (p->audio_track, af);
@@ -285,12 +298,11 @@ decode_audio (GeglOperation *operation,
             p->prevapts = pkt.pts * av_q2d (p->audio_stream->time_base);
           }
         }
-      av_free_packet (&pkt);
+      av_packet_unref (&pkt);
     }
   return 0;
 }
 
-
 static int
 decode_frame (GeglOperation *operation,
               glong          frame)
@@ -324,7 +336,7 @@ decode_frame (GeglOperation *operation,
       fprintf (stderr, "video seek error!\n");
     else
       avcodec_flush_buffers (p->video_stream->codec);
-    
+
     prevframe = -1;
   }
 
@@ -338,10 +350,10 @@ decode_frame (GeglOperation *operation,
 
           do
           {
-            av_free_packet (&pkt);
+            av_packet_unref (&pkt);
             if (av_read_frame (p->video_fcontext, &pkt) < 0)
             {
-              av_free_packet (&pkt);
+              av_packet_unref (&pkt);
               return -1;
             }
           }
@@ -374,10 +386,11 @@ AV_TIME_BASE_Q) * 1.0 / AV_TIME_BASE ;
                decodeframe = roundf ( p->prevpts * o->frame_rate);
              }
           }
-
+#if 0
           if (decoded_bytes != pkt.size)
             fprintf (stderr, "bytes left!\n");
-          av_free_packet (&pkt);
+#endif
+          av_packet_unref (&pkt);
         }
       while (!got_picture);
     }
@@ -399,17 +412,20 @@ prepare (GeglOperation *operation)
 
   g_assert (o->user_data != NULL);
 
-  gegl_operation_set_format (operation, "output", babl_format ("R'G'B'A u8"));
-
+  gegl_operation_set_format (operation, "output", babl_format ("R'G'B' u8"));
 
   if (!p->loadedfilename ||
-      strcmp (p->loadedfilename, o->path))
+      strcmp (p->loadedfilename, o->path) ||
+       p->prevframe > o->frame  /* a bit heavy handed, but improves consistency */
+      )
     {
       gint i;
+      gchar dereferenced_path[PATH_MAX];
       gint err;
 
       ff_cleanup (o);
-      err = avformat_open_input(&p->video_fcontext, o->path, NULL, 0);
+      realpath (o->path, dereferenced_path);
+      err = avformat_open_input(&p->video_fcontext, dereferenced_path, NULL, 0);
       if (err < 0)
         {
           print_error (o->path, err);
@@ -466,25 +482,11 @@ prepare (GeglOperation *operation)
               }
         }
 
-      p->video_stream->codec->err_recognition = AV_EF_IGNORE_ERR | AV_EF_BITSTREAM | AV_EF_BUFFER;
+      p->video_stream->codec->err_recognition = AV_EF_IGNORE_ERR |
+                                                AV_EF_BITSTREAM |
+                                                AV_EF_BUFFER;
       p->video_stream->codec->workaround_bugs = FF_BUG_AUTODETECT;
 
-#if 1
- //     p->video_stream->codec->error_concealment = 0;
-#else
-      p->video_stream->codec->error_concealment = FF_EC_DEBLOCK | FF_EC_GUESS_MVS | FF_EC_FAVOR_INTER;
-
-#endif
-
-#if 0
-      p->video_stream->codec->idct_algo = FF_IDCT_SIMPLEAUTO;
-
-      p->video_stream->codec->thread_count = 0;
-      p->video_stream->codec->thread_type = FF_THREAD_SLICE;
-      /* XXX: permits slice parallell decode, at expense of h264 compliance of output */
-      p->video_stream->codec->flags2 = AV_CODEC_FLAG2_FAST;
-#endif
-
       if (p->video_codec == NULL)
           g_warning ("video codec not found");
 
@@ -526,7 +528,7 @@ prepare (GeglOperation *operation)
 	o->frames = p->video_fcontext->duration * o->frame_rate / AV_TIME_BASE;
         /* make second guess for things like luxo */
 	if (o->frames < 1)
-          o->frames = 1000;
+          o->frames = 23;
       }
 #if 0
       {
@@ -539,25 +541,24 @@ prepare (GeglOperation *operation)
         fprintf (stdout, "duration: %02i:%02i:%02i\n", h, m, s);
       }
 #endif
+
+    p->codec_delay = p->video_stream->codec->delay;
+  
+    if (!strcmp (o->video_codec, "mpeg1video"))
+      p->codec_delay = 1;
+    else if (!strcmp (o->video_codec, "h264"))
+    {
+      if (strstr (p->video_fcontext->filename, ".mp4") ||
+          strstr (p->video_fcontext->filename, ".MP4"))  /* XXX: too hacky, isn't there an avformat thing to use?,
+ or perhaps we can measure this when decoding the first frame.
+ */
+        p->codec_delay = 3;
+      else
+        p->codec_delay = 0;
     }
 
-  /* is there a way to compute this from internals
-   */
-  if (!strcmp (o->video_codec, "mpeg1video"))
-    p->codec_delay = 1;
-  else if (!strcmp (o->video_codec, "mpeg4"))
-    p->codec_delay = 2;
-  else if (!strcmp (o->video_codec, "vp9"))
-    p->codec_delay = 2;
-  else if (!strcmp (o->video_codec, "theora"))
-    p->codec_delay = 2;
-  else if (!strcmp (o->video_codec, "h264"))
-    p->codec_delay = 0;
-  else
-    {
-      p->codec_delay = 0;
-      fprintf (stderr, "[assuming no codec_delay for: %s]", o->video_codec);
-    }
+    clear_audio_track (o);
+  }
 }
 
 static GeglRectangle
@@ -642,11 +643,31 @@ static void get_sample_data (Priv *p, long sample_no, float *left, float *right)
         return;
       }
   }
-  //fprintf (stderr, "didn't find audio sample\n");
   *left  = 0;
   *right = 0;
 }
 
+static AVFrame *
+alloc_picture (int pix_fmt, int width, int height)
+{
+  AVFrame  *picture;
+  uint8_t  *picture_buf;
+  int       size;
+
+  picture = av_frame_alloc ();
+  if (!picture)
+    return NULL;
+  size = av_image_get_buffer_size (pix_fmt, width + 1, height + 1, 16);
+  picture_buf = malloc (size);
+  if (!picture_buf)
+    {
+      av_free (picture);
+      return NULL;
+    }
+  av_image_fill_arrays (picture->data, picture->linesize, picture_buf, pix_fmt, width, height, 1);
+  return picture;
+}
+
 static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *output,
@@ -659,13 +680,9 @@ process (GeglOperation       *operation,
   {
     if (p->video_fcontext && !decode_frame (operation, o->frame))
       {
-        guchar *buf;
-        gint    pxsize = 4;
-        gint    x,y;
-
         long sample_start = 0;
 
-	if (p->audio_stream && p->audio_stream->codec) // XXX: remove second clause
+	if (p->audio_stream) 
         {
           int sample_count;
           gegl_audio_fragment_set_sample_rate (o->audio, p->audio_stream->codec->sample_rate);
@@ -688,42 +705,26 @@ process (GeglOperation       *operation,
           }
         }
 	
-        buf = g_new (guchar, p->width * p->height * pxsize);
-
-        for (y=0; y < p->height; y++)
-          {
-            guchar       *dst  = buf                    + y     * p->width * 4;
-            const guchar *ysrc = p->lavc_frame->data[0] + y     * p->lavc_frame->linesize[0];
-            const guchar *usrc = p->lavc_frame->data[1] + y / 2 * p->lavc_frame->linesize[1];
-            const guchar *vsrc = p->lavc_frame->data[2] + y / 2 * p->lavc_frame->linesize[2];
-
-            for (x=0;x < p->width; x++)
-              {
-                gint R,G,B;
-#ifndef byteclamp
-#define byteclamp(j) j=j<0?0:j>255?255:j
-#endif
-#define YUV82RGB8(Y,U,V,R,G,B)do{\
-                R= ((Y<<15)                 + 37355*(V-128))>>15;\
-                G= ((Y<<15) -12911* (U-128) - 19038*(V-128))>>15;\
-                B= ((Y<<15) +66454* (U-128)                )>>15;\
-                byteclamp(R);\
-                byteclamp(G);\
-                byteclamp(B);\
-              } while(0)
-              YUV82RGB8 (*ysrc, *usrc, *vsrc, R, G, B);
-              *(unsigned int *) dst = R + G * 256 + B * 256 * 256 + 0xff000000;
-              dst  += 4;
-              usrc += x%2;
-              vsrc += x%2;
-              ysrc ++;
-              }
-          }
+        if (p->video_stream->codec->pix_fmt == AV_PIX_FMT_RGB24)
         {
           GeglRectangle extent = {0,0,p->width,p->height};
-          gegl_buffer_set (output, &extent, 0, NULL, buf, GEGL_AUTO_ROWSTRIDE);
+          gegl_buffer_set (output, &extent, 0, babl_format("R'G'B' u8"), p->lavc_frame->data[0], GEGL_AUTO_ROWSTRIDE);
+        }
+        else
+        {
+          struct SwsContext *img_convert_ctx;
+          GeglRectangle extent = {0,0,p->width,p->height};
+
+          img_convert_ctx = sws_getContext(p->width, p->height, p->video_stream->codec->pix_fmt,
+                                           p->width, p->height, AV_PIX_FMT_RGB24,
+                                           SWS_BICUBIC, NULL, NULL, NULL);
+          if (!p->rgb_frame)
+            p->rgb_frame = alloc_picture (AV_PIX_FMT_RGB24, p->width, p->height);
+          sws_scale (img_convert_ctx, (void*)p->lavc_frame->data,
+                     p->lavc_frame->linesize, 0, p->height, p->rgb_frame->data, p->rgb_frame->linesize);
+          gegl_buffer_set (output, &extent, 0, babl_format("R'G'B' u8"), p->rgb_frame->data[0], GEGL_AUTO_ROWSTRIDE);
+          sws_freeContext (img_convert_ctx);
         }
-        g_free (buf);
       }
   }
   return  TRUE;
