@@ -19,6 +19,11 @@ static const char* op_c_source =
 "                                                                              \n"
 "#include \"config.h\"                                                         \n"
 "#include <glib/gi18n-lib.h>                                                   \n"
+"#include <stdlib.h>                                                           \n"
+"                                                                              \n"
+"#ifdef G_OS_WIN32                                                             \n"
+"#define realpath(a,b) _fullpath(b,a,_MAX_PATH)                                \n"
+"#endif                                                                        \n"
 "                                                                              \n"
 "#ifdef GEGL_PROPERTIES                                                        \n"
 "                                                                              \n"
@@ -54,8 +59,14 @@ static const char* op_c_source =
 "                                                                              \n"
 "#include \"gegl-op.h\"                                                        \n"
 "#include <errno.h>                                                            \n"
+"#include <limits.h>                                                           \n"
+"#include <stdlib.h>                                                           \n"
 "                                                                              \n"
+"#include <libavutil/avutil.h>                                                 \n"
+"#include <libavutil/imgutils.h>                                               \n"
 "#include <libavformat/avformat.h>                                             \n"
+"#include <libswscale/swscale.h>                                               \n"
+"                                                                              \n"
 "                                                                              \n"
 "typedef struct                                                                \n"
 "{                                                                             \n"
@@ -82,6 +93,7 @@ static const char* op_c_source =
 "  AVStream        *audio_stream;                                              \n"
 "  AVCodec         *video_codec;                                               \n"
 "  AVFrame         *lavc_frame;                                                \n"
+"  AVFrame         *rgb_frame;                                                 \n"
 "  glong            prevframe;      /* previously decoded frame number */      \n"
 "  gdouble          prevpts;        /* timestamp in seconds of last decoded frame */\n"
 "                                                                              \n"
@@ -117,6 +129,7 @@ static const char* op_c_source =
 "      g_object_unref (p->audio_track->data);                                  \n"
 "      p->audio_track = g_list_remove (p->audio_track, p->audio_track->data);  \n"
 "    }                                                                         \n"
+"  p->prevapts = 0.0;                                                          \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static void                                                                   \n"
@@ -136,12 +149,15 @@ static const char* op_c_source =
 "        avformat_close_input(&p->video_fcontext);                             \n"
 "      if (p->audio_fcontext)                                                  \n"
 "        avformat_close_input(&p->audio_fcontext);                             \n"
+"      if (p->rgb_frame)                                                       \n"
+"        av_free (p->rgb_frame);                                               \n"
 "      if (p->lavc_frame)                                                      \n"
 "        av_free (p->lavc_frame);                                              \n"
 "                                                                              \n"
 "      p->video_fcontext = NULL;                                               \n"
 "      p->audio_fcontext = NULL;                                               \n"
 "      p->lavc_frame = NULL;                                                   \n"
+"      p->rgb_frame = NULL;                                                    \n"
 "      p->loadedfilename = NULL;                                               \n"
 "    }                                                                         \n"
 "}                                                                             \n"
@@ -170,6 +186,7 @@ static const char* op_c_source =
 "  p->loadedfilename = g_strdup (\"\");                                        \n"
 "                                                                              \n"
 "  ff_cleanup (o);                                                             \n"
+"  av_log_set_level (AV_LOG_WARNING);                                          \n"
 "}                                                                             \n"
 "                                                                              \n"
 "/* maintain list of audio samples */                                          \n"
@@ -181,22 +198,19 @@ static const char* op_c_source =
 "  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
 "  Priv       *p = (Priv*)o->user_data;                                        \n"
 "                                                                              \n"
-"  pts1 -= 15.0;                                                               \n"
+"  pts1 -= 2.0;                                                                \n"
 "  if (pts1 < 0.0)pts1 = 0.0;                                                  \n"
 "                                                                              \n"
-"  /* figure out which frame we should start decoding at */                    \n"
-"  //fprintf (stderr, \"%f %f\\n\", p->prevapts, pts2);                        \n"
-"  if(0){                                                                      \n"
-"  //int64_t seek_target = av_rescale_q ((pts1) * AV_TIME_BASE, AV_TIME_BASE_Q, p->audio_stream->time_base);\n"
-"  int64_t seek_target = pts1 * AV_TIME_BASE;                                  \n"
+"  if(pts1 - 15.0 > p->prevapts){                                              \n"
+"  int64_t seek_target = av_rescale_q (pts1 * AV_TIME_BASE, AV_TIME_BASE_Q, p->audio_stream->time_base);\n"
+"     clear_audio_track (o);                                                   \n"
+"     p->prevapts = 0.0;                                                       \n"
 "                                                                              \n"
-"#if 0                                                                         \n"
-"  clear_audio_track (o);                                                      \n"
-"#endif                                                                        \n"
-"  p->prevapts = 0.0;                                                          \n"
-"                                                                              \n"
-"    if (av_seek_frame (p->audio_fcontext, -1, seek_target, (AVSEEK_FLAG_BACKWARD)) < 0)\n"
+"     if (av_seek_frame (p->audio_fcontext, p->audio_stream->index, seek_target, (AVSEEK_FLAG_BACKWARD)) < 0)\n"
 "      fprintf (stderr, \"audio seek error!\\n\");                             \n"
+"     else                                                                     \n"
+"      avcodec_flush_buffers (p->audio_stream->codec);                         \n"
+"                                                                              \n"
 "  }                                                                           \n"
 "                                                                              \n"
 "  while (p->prevapts <= pts2)                                                 \n"
@@ -206,7 +220,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "      if (av_read_frame (p->audio_fcontext, &pkt) < 0)                        \n"
 "         {                                                                    \n"
-"           av_free_packet (&pkt);                                             \n"
+"           av_packet_unref (&pkt);                                            \n"
 "           return -1;                                                         \n"
 "         }                                                                    \n"
 "      if (pkt.stream_index==p->audio_index && p->audio_stream)                \n"
@@ -234,8 +248,6 @@ static const char* op_c_source =
 "               GeglAudioFragment *af = gegl_audio_fragment_new (o->audio_sample_rate, channels,\n"
 "                          AV_CH_LAYOUT_STEREO, samples_left);                 \n"
 "//);                                                                          \n"
-"               //af->channels = MIN(p->audio_stream->codec->channels, GEGL_MAX_AUDIO_CHANNELS);\n"
-"                                                                              \n"
 "               switch (p->audio_stream->codec->sample_fmt)                    \n"
 "               {                                                              \n"
 "                 case AV_SAMPLE_FMT_FLT:                                      \n"
@@ -274,7 +286,8 @@ static const char* op_c_source =
 "                  g_warning (\"undealt with sample format\\n\");              \n"
 "                }                                                             \n"
 "                gegl_audio_fragment_set_sample_count (af, sample_count);      \n"
-"                gegl_audio_fragment_set_pos (af, p->audio_pos);               \n"
+"                gegl_audio_fragment_set_pos (af,                              \n"
+"  (long int)av_rescale_q ((pkt.pts), p->audio_stream->time_base, AV_TIME_BASE_Q) * o->audio_sample_rate /AV_TIME_BASE);\n"
 "                                                                              \n"
 "                p->audio_pos += sample_count;                                 \n"
 "                p->audio_track = g_list_append (p->audio_track, af);          \n"
@@ -286,12 +299,11 @@ static const char* op_c_source =
 "            p->prevapts = pkt.pts * av_q2d (p->audio_stream->time_base);      \n"
 "          }                                                                   \n"
 "        }                                                                     \n"
-"      av_free_packet (&pkt);                                                  \n"
+"      av_packet_unref (&pkt);                                                 \n"
 "    }                                                                         \n"
 "  return 0;                                                                   \n"
 "}                                                                             \n"
 "                                                                              \n"
-"                                                                              \n"
 "static int                                                                    \n"
 "decode_frame (GeglOperation *operation,                                       \n"
 "              glong          frame)                                           \n"
@@ -339,10 +351,10 @@ static const char* op_c_source =
 "                                                                              \n"
 "          do                                                                  \n"
 "          {                                                                   \n"
-"            av_free_packet (&pkt);                                            \n"
+"            av_packet_unref (&pkt);                                           \n"
 "            if (av_read_frame (p->video_fcontext, &pkt) < 0)                  \n"
 "            {                                                                 \n"
-"              av_free_packet (&pkt);                                          \n"
+"              av_packet_unref (&pkt);                                         \n"
 "              return -1;                                                      \n"
 "            }                                                                 \n"
 "          }                                                                   \n"
@@ -375,10 +387,11 @@ static const char* op_c_source =
 "               decodeframe = roundf ( p->prevpts * o->frame_rate);            \n"
 "             }                                                                \n"
 "          }                                                                   \n"
-"                                                                              \n"
+"#if 0                                                                         \n"
 "          if (decoded_bytes != pkt.size)                                      \n"
 "            fprintf (stderr, \"bytes left!\\n\");                             \n"
-"          av_free_packet (&pkt);                                              \n"
+"#endif                                                                        \n"
+"          av_packet_unref (&pkt);                                             \n"
 "        }                                                                     \n"
 "      while (!got_picture);                                                   \n"
 "    }                                                                         \n"
@@ -400,17 +413,20 @@ static const char* op_c_source =
 "                                                                              \n"
 "  g_assert (o->user_data != NULL);                                            \n"
 "                                                                              \n"
-"  gegl_operation_set_format (operation, \"output\", babl_format (\"R'G'B'A u8\"));\n"
-"                                                                              \n"
+"  gegl_operation_set_format (operation, \"output\", babl_format (\"R'G'B' u8\"));\n"
 "                                                                              \n"
 "  if (!p->loadedfilename ||                                                   \n"
-"      strcmp (p->loadedfilename, o->path))                                    \n"
+"      strcmp (p->loadedfilename, o->path) ||                                  \n"
+"       p->prevframe > o->frame  /* a bit heavy handed, but improves consistency */\n"
+"      )                                                                       \n"
 "    {                                                                         \n"
 "      gint i;                                                                 \n"
+"      gchar dereferenced_path[PATH_MAX];                                      \n"
 "      gint err;                                                               \n"
 "                                                                              \n"
 "      ff_cleanup (o);                                                         \n"
-"      err = avformat_open_input(&p->video_fcontext, o->path, NULL, 0);        \n"
+"      realpath (o->path, dereferenced_path);                                  \n"
+"      err = avformat_open_input(&p->video_fcontext, dereferenced_path, NULL, 0);\n"
 "      if (err < 0)                                                            \n"
 "        {                                                                     \n"
 "          print_error (o->path, err);                                         \n"
@@ -467,25 +483,11 @@ static const char* op_c_source =
 "              }                                                               \n"
 "        }                                                                     \n"
 "                                                                              \n"
-"      p->video_stream->codec->err_recognition = AV_EF_IGNORE_ERR | AV_EF_BITSTREAM | AV_EF_BUFFER;\n"
+"      p->video_stream->codec->err_recognition = AV_EF_IGNORE_ERR |            \n"
+"                                                AV_EF_BITSTREAM |             \n"
+"                                                AV_EF_BUFFER;                 \n"
 "      p->video_stream->codec->workaround_bugs = FF_BUG_AUTODETECT;            \n"
 "                                                                              \n"
-"#if 1                                                                         \n"
-" //     p->video_stream->codec->error_concealment = 0;                        \n"
-"#else                                                                         \n"
-"      p->video_stream->codec->error_concealment = FF_EC_DEBLOCK | FF_EC_GUESS_MVS | FF_EC_FAVOR_INTER;\n"
-"                                                                              \n"
-"#endif                                                                        \n"
-"                                                                              \n"
-"#if 0                                                                         \n"
-"      p->video_stream->codec->idct_algo = FF_IDCT_SIMPLEAUTO;                 \n"
-"                                                                              \n"
-"      p->video_stream->codec->thread_count = 0;                               \n"
-"      p->video_stream->codec->thread_type = FF_THREAD_SLICE;                  \n"
-"      /* XXX: permits slice parallell decode, at expense of h264 compliance of output */\n"
-"      p->video_stream->codec->flags2 = AV_CODEC_FLAG2_FAST;                   \n"
-"#endif                                                                        \n"
-"                                                                              \n"
 "      if (p->video_codec == NULL)                                             \n"
 "          g_warning (\"video codec not found\");                              \n"
 "                                                                              \n"
@@ -527,7 +529,7 @@ static const char* op_c_source =
 "	o->frames = p->video_fcontext->duration * o->frame_rate / AV_TIME_BASE;      \n"
 "        /* make second guess for things like luxo */                          \n"
 "	if (o->frames < 1)                                                           \n"
-"          o->frames = 1000;                                                   \n"
+"          o->frames = 23;                                                     \n"
 "      }                                                                       \n"
 "#if 0                                                                         \n"
 "      {                                                                       \n"
@@ -540,25 +542,24 @@ static const char* op_c_source =
 "        fprintf (stdout, \"duration: %02i:%02i:%02i\\n\", h, m, s);           \n"
 "      }                                                                       \n"
 "#endif                                                                        \n"
+"                                                                              \n"
+"    p->codec_delay = p->video_stream->codec->delay;                           \n"
+"                                                                              \n"
+"    if (!strcmp (o->video_codec, \"mpeg1video\"))                             \n"
+"      p->codec_delay = 1;                                                     \n"
+"    else if (!strcmp (o->video_codec, \"h264\"))                              \n"
+"    {                                                                         \n"
+"      if (strstr (p->video_fcontext->filename, \".mp4\") ||                   \n"
+"          strstr (p->video_fcontext->filename, \".MP4\"))  /* XXX: too hacky, isn't there an avformat thing to use?,\n"
+" or perhaps we can measure this when decoding the first frame.                \n"
+" */                                                                           \n"
+"        p->codec_delay = 3;                                                   \n"
+"      else                                                                    \n"
+"        p->codec_delay = 0;                                                   \n"
 "    }                                                                         \n"
 "                                                                              \n"
-"  /* is there a way to compute this from internals                            \n"
-"   */                                                                         \n"
-"  if (!strcmp (o->video_codec, \"mpeg1video\"))                               \n"
-"    p->codec_delay = 1;                                                       \n"
-"  else if (!strcmp (o->video_codec, \"mpeg4\"))                               \n"
-"    p->codec_delay = 2;                                                       \n"
-"  else if (!strcmp (o->video_codec, \"vp9\"))                                 \n"
-"    p->codec_delay = 2;                                                       \n"
-"  else if (!strcmp (o->video_codec, \"theora\"))                              \n"
-"    p->codec_delay = 2;                                                       \n"
-"  else if (!strcmp (o->video_codec, \"h264\"))                                \n"
-"    p->codec_delay = 0;                                                       \n"
-"  else                                                                        \n"
-"    {                                                                         \n"
-"      p->codec_delay = 0;                                                     \n"
-"      fprintf (stderr, \"[assuming no codec_delay for: %s]\", o->video_codec);\n"
-"    }                                                                         \n"
+"    clear_audio_track (o);                                                    \n"
+"  }                                                                           \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static GeglRectangle                                                          \n"
@@ -643,11 +644,31 @@ static const char* op_c_source =
 "        return;                                                               \n"
 "      }                                                                       \n"
 "  }                                                                           \n"
-"  //fprintf (stderr, \"didn't find audio sample\\n\");                        \n"
 "  *left  = 0;                                                                 \n"
 "  *right = 0;                                                                 \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static AVFrame *                                                              \n"
+"alloc_picture (int pix_fmt, int width, int height)                            \n"
+"{                                                                             \n"
+"  AVFrame  *picture;                                                          \n"
+"  uint8_t  *picture_buf;                                                      \n"
+"  int       size;                                                             \n"
+"                                                                              \n"
+"  picture = av_frame_alloc ();                                                \n"
+"  if (!picture)                                                               \n"
+"    return NULL;                                                              \n"
+"  size = av_image_get_buffer_size (pix_fmt, width + 1, height + 1, 16);       \n"
+"  picture_buf = malloc (size);                                                \n"
+"  if (!picture_buf)                                                           \n"
+"    {                                                                         \n"
+"      av_free (picture);                                                      \n"
+"      return NULL;                                                            \n"
+"    }                                                                         \n"
+"  av_image_fill_arrays (picture->data, picture->linesize, picture_buf, pix_fmt, width, height, 1);\n"
+"  return picture;                                                             \n"
+"}                                                                             \n"
+"                                                                              \n"
 "static gboolean                                                               \n"
 "process (GeglOperation       *operation,                                      \n"
 "         GeglBuffer          *output,                                         \n"
@@ -660,13 +681,9 @@ static const char* op_c_source =
 "  {                                                                           \n"
 "    if (p->video_fcontext && !decode_frame (operation, o->frame))             \n"
 "      {                                                                       \n"
-"        guchar *buf;                                                          \n"
-"        gint    pxsize = 4;                                                   \n"
-"        gint    x,y;                                                          \n"
-"                                                                              \n"
 "        long sample_start = 0;                                                \n"
 "                                                                              \n"
-"	if (p->audio_stream && p->audio_stream->codec) // XXX: remove second clause  \n"
+"	if (p->audio_stream)                                                         \n"
 "        {                                                                     \n"
 "          int sample_count;                                                   \n"
 "          gegl_audio_fragment_set_sample_rate (o->audio, p->audio_stream->codec->sample_rate);\n"
@@ -689,42 +706,26 @@ static const char* op_c_source =
 "          }                                                                   \n"
 "        }                                                                     \n"
 "                                                                              \n"
-"        buf = g_new (guchar, p->width * p->height * pxsize);                  \n"
-"                                                                              \n"
-"        for (y=0; y < p->height; y++)                                         \n"
-"          {                                                                   \n"
-"            guchar       *dst  = buf                    + y     * p->width * 4;\n"
-"            const guchar *ysrc = p->lavc_frame->data[0] + y     * p->lavc_frame->linesize[0];\n"
-"            const guchar *usrc = p->lavc_frame->data[1] + y / 2 * p->lavc_frame->linesize[1];\n"
-"            const guchar *vsrc = p->lavc_frame->data[2] + y / 2 * p->lavc_frame->linesize[2];\n"
-"                                                                              \n"
-"            for (x=0;x < p->width; x++)                                       \n"
-"              {                                                               \n"
-"                gint R,G,B;                                                   \n"
-"#ifndef byteclamp                                                             \n"
-"#define byteclamp(j) j=j<0?0:j>255?255:j                                      \n"
-"#endif                                                                        \n"
-"#define YUV82RGB8(Y,U,V,R,G,B)do{\\                                           \n"
-"                R= ((Y<<15)                 + 37355*(V-128))>>15;\\           \n"
-"                G= ((Y<<15) -12911* (U-128) - 19038*(V-128))>>15;\\           \n"
-"                B= ((Y<<15) +66454* (U-128)                )>>15;\\           \n"
-"                byteclamp(R);\\                                               \n"
-"                byteclamp(G);\\                                               \n"
-"                byteclamp(B);\\                                               \n"
-"              } while(0)                                                      \n"
-"              YUV82RGB8 (*ysrc, *usrc, *vsrc, R, G, B);                       \n"
-"              *(unsigned int *) dst = R + G * 256 + B * 256 * 256 + 0xff000000;\n"
-"              dst  += 4;                                                      \n"
-"              usrc += x%2;                                                    \n"
-"              vsrc += x%2;                                                    \n"
-"              ysrc ++;                                                        \n"
-"              }                                                               \n"
-"          }                                                                   \n"
+"        if (p->video_stream->codec->pix_fmt == AV_PIX_FMT_RGB24)              \n"
 "        {                                                                     \n"
 "          GeglRectangle extent = {0,0,p->width,p->height};                    \n"
-"          gegl_buffer_set (output, &extent, 0, NULL, buf, GEGL_AUTO_ROWSTRIDE);\n"
+"          gegl_buffer_set (output, &extent, 0, babl_format(\"R'G'B' u8\"), p->lavc_frame->data[0], GEGL_AUTO_ROWSTRIDE);\n"
+"        }                                                                     \n"
+"        else                                                                  \n"
+"        {                                                                     \n"
+"          struct SwsContext *img_convert_ctx;                                 \n"
+"          GeglRectangle extent = {0,0,p->width,p->height};                    \n"
+"                                                                              \n"
+"          img_convert_ctx = sws_getContext(p->width, p->height, p->video_stream->codec->pix_fmt,\n"
+"                                           p->width, p->height, AV_PIX_FMT_RGB24,\n"
+"                                           SWS_BICUBIC, NULL, NULL, NULL);    \n"
+"          if (!p->rgb_frame)                                                  \n"
+"            p->rgb_frame = alloc_picture (AV_PIX_FMT_RGB24, p->width, p->height);\n"
+"          sws_scale (img_convert_ctx, (void*)p->lavc_frame->data,             \n"
+"                     p->lavc_frame->linesize, 0, p->height, p->rgb_frame->data, p->rgb_frame->linesize);\n"
+"          gegl_buffer_set (output, &extent, 0, babl_format(\"R'G'B' u8\"), p->rgb_frame->data[0], GEGL_AUTO_ROWSTRIDE);\n"
+"          sws_freeContext (img_convert_ctx);                                  \n"
 "        }                                                                     \n"
-"        g_free (buf);                                                         \n"
 "      }                                                                       \n"
 "  }                                                                           \n"
 "  return  TRUE;                                                               \n"
