@@ -22,31 +22,58 @@
 
 #include <glib/gi18n-lib.h>
 
+//#define USE_FINE_GRAINDED_FFMPEG 1
+
 #ifdef GEGL_PROPERTIES
 
 property_string (path, _("File"), "/tmp/fnord.ogv")
     description (_("Target path and filename, use '-' for stdout."))
 
+property_audio_fragment (audio, _("audio"), 0)
+property_string (audio_codec, _("Audio codec"), "auto")
+   description (_("Audio codec to use, or auto to use a good default based on container format."))
+property_int (audio_sample_rate, _("audio sample rate"), -1)
+    description (_("-1 means autodetect on first audio fragment"))
+
+property_int (audio_bit_rate, _("audio bitrate in kb/s"), 64)
+    description (_("Target encoded video bitrate in kb/s"))
+
 property_double (frame_rate, _("Frames/second"), 25.0)
     value_range (0.0, 100.0)
-#if 0
-property_string (audio_codec, _("Audio codec"), "auto")
-property_int (audio_bit_rate, _("Audio bitrate"), 810000)
-property_string (video_codec,   _("Video codec"),   "auto")
-property_int (video_bit_rate, _("video bitrate"), 810000)
-    value_range (0.0, 500000000.0)
-property_double (video_bit_rate_tolerance, _("video bitrate"), 1000.0)
-property_int    (video_global_quality, _("global quality"), 255)
-property_int    (compression_level,    _("compression level"), 255)
-property_int    (noise_reduction,      _("noise reduction strength"), 0)
-property_int    (gop_size,             _("the number of frames in a group of pictures, 0 for keyframe only"), 16)
-property_int    (key_int_min,          _("the minimum number of frames in a group of pictures, 0 for keyframe only"), 1)
-property_int    (max_b_frames,         _("maximum number of consequetive b frames"), 3)
+
+property_string (video_codec, _("Video codec"), "auto")
+   description (_("Video codec to use, or auto to use a good default based on container format."))
+property_int (video_bit_rate, _("video bitrate in kb/s"), 128)
+    description (_("Target encoded video bitrate in kb/s"))
+property_int (video_bufsize, _("Video bufsize"), 0)
+
+property_string (container_format, _("Container format"), "auto")
+   description (_("Container format to use, or auto to autodetect based on file extension."))
+
+#ifdef USE_FINE_GRAINED_FFMPEG
+property_int (global_quality, _("global quality"), 0)
+property_int (noise_reduction, _("noise reduction"), 0)
+property_int (scenechange_threshold, _("scenechange threshold"), 0)
+property_int (video_bit_rate_min, _("video bitrate min"), 0)
+property_int (video_bit_rate_max, _("video bitrate max"), 0)
+property_int (video_bit_rate_tolerance, _("video bitrate tolerance"), -1)
+
+property_int (keyint_min, _("keyint-min"), 0)
+property_int (trellis, _("trellis"), 0)
+property_int (qmin, _("qmin"), 0)
+property_int (qmax, _("qmax"), 0)
+property_int (max_qdiff, _("max_qdiff"), 0)
+property_int (me_range, _("me_range"), 0)
+property_int (max_b_frames, _("max_b_frames"), 0)
+property_int (gop_size, _("gop-size"), 0)
+property_double (qcompress, _("qcompress"), 0.0)
+property_double (qblur, _("qblur"), 0.0)
+property_double (i_quant_factor, _("i-quant-factor"), 0.0)
+property_double (i_quant_offset, _("i-quant-offset"), 0.0)
+property_int (me_subpel_quality, _("me-subpel-quality"), 0)
 #endif
 
 
-property_audio_fragment (audio, _("audio"), 0)
-
 #else
 
 #define GEGL_OP_SINK
@@ -75,7 +102,6 @@ typedef struct
   AVFrame  *picture, *tmp_picture;
   uint8_t  *video_outbuf;
   int       frame_count, video_outbuf_size;
-  int       audio_sample_rate;
 
     /** the rest is for audio handling within oxide, note that the interface
      * used passes all used functions in the oxide api through the reg_sym api
@@ -90,7 +116,7 @@ typedef struct
   uint32_t  fragment_samples;
   uint32_t  fragment_size;
 
-  int       buffer_size;
+  int       bufsize;
   int       buffer_read_pos;
   int       buffer_write_pos;
   uint8_t  *buffer; 
@@ -116,6 +142,34 @@ clear_audio_track (GeglProperties *o)
     }
 }
 
+static int
+samples_per_frame (int    frame,
+                   double frame_rate,
+                   int    sample_rate,
+                   long  *start)
+{
+  double osamples;
+  double samples = 0;
+  int f = 0;
+
+  if (fabs(fmod (sample_rate, frame_rate)) < 0.0001)
+  {
+    if (start)
+      *start = (sample_rate / frame_rate) * frame;
+    return sample_rate / frame_rate;
+  }
+
+  for (f = 0; f < frame; f++) 
+  {
+    samples += sample_rate / frame_rate;
+  }
+  osamples = samples;
+  samples += sample_rate / frame_rate;
+  if (start)
+     (*start) = ceil(osamples);
+  return ceil(samples)-ceil(osamples);
+}
+
 static void get_sample_data (Priv *p, long sample_no, float *left, float *right)
 {
   int to_remove = 0;
@@ -189,7 +243,9 @@ init (GeglProperties *o)
   p->audio_pos = 0;
   p->audio_read_pos = 0;
 
-  p->audio_sample_rate = -1; /* only do this if it hasn't been manually set? */
+  o->audio_sample_rate = -1; /* only do this if it hasn't been manually set? */
+
+  av_log_set_level (AV_LOG_WARNING);
 }
 
 static void close_video       (Priv            *p,
@@ -237,7 +293,6 @@ add_audio_stream (GeglProperties *o, AVFormatContext * oc, int codec_id)
 static void
 open_audio (GeglProperties *o, AVFormatContext * oc, AVStream * st)
 {
-  Priv *p = (Priv*)o->user_data;
   AVCodecContext *c;
   AVCodec  *codec;
   int i;
@@ -251,10 +306,10 @@ open_audio (GeglProperties *o, AVFormatContext * oc, AVStream * st)
       fprintf (stderr, "codec not found\n");
       exit (1);
     }
-  c->bit_rate = 64000;
+  c->bit_rate = o->audio_bit_rate * 1000;
   c->sample_fmt = codec->sample_fmts ? codec->sample_fmts[0] : AV_SAMPLE_FMT_FLTP;
 
-  if (p->audio_sample_rate == -1)
+  if (o->audio_sample_rate == -1)
   {
     if (o->audio)
     {
@@ -262,10 +317,10 @@ open_audio (GeglProperties *o, AVFormatContext * oc, AVStream * st)
       {
         gegl_audio_fragment_set_sample_rate (o->audio, 48000); // XXX: should skip adding audiostream instead
       }
-      p->audio_sample_rate = gegl_audio_fragment_get_sample_rate (o->audio);
+      o->audio_sample_rate = gegl_audio_fragment_get_sample_rate (o->audio);
     }
   }
-  c->sample_rate = p->audio_sample_rate;
+  c->sample_rate = o->audio_sample_rate;
   c->channel_layout = AV_CH_LAYOUT_STEREO;
   c->channels = 2;
 
@@ -275,12 +330,12 @@ open_audio (GeglProperties *o, AVFormatContext * oc, AVStream * st)
     c->sample_rate = codec->supported_samplerates[0];
     for (i = 0; codec->supported_samplerates[i]; i++)
     {
-      if (codec->supported_samplerates[i] == p->audio_sample_rate)
-         c->sample_rate = p->audio_sample_rate;
+      if (codec->supported_samplerates[i] == o->audio_sample_rate)
+         c->sample_rate = o->audio_sample_rate;
     }
   }
   //st->time_base = (AVRational){1, c->sample_rate};
-  st->time_base = (AVRational){1, p->audio_sample_rate};
+  st->time_base = (AVRational){1, o->audio_sample_rate};
 
   c->strict_std_compliance = FF_COMPLIANCE_EXPERIMENTAL; // ffmpeg AAC is not quite stable yet
 
@@ -333,10 +388,12 @@ write_audio_frame (GeglProperties *o, AVFormatContext * oc, AVStream * st)
   }
 
   /* first we add incoming frames audio samples */
+  if (o->audio)
   {
     int i;
-    int sample_count = gegl_audio_fragment_get_sample_count (o->audio);
-    GeglAudioFragment *af = gegl_audio_fragment_new (gegl_audio_fragment_get_sample_rate (o->audio),
+    GeglAudioFragment *af;
+    sample_count = gegl_audio_fragment_get_sample_count (o->audio);
+    af = gegl_audio_fragment_new (gegl_audio_fragment_get_sample_rate (o->audio),
                                                      gegl_audio_fragment_get_channels (o->audio),
                                                      gegl_audio_fragment_get_channel_layout (o->audio),
                                                      sample_count);
@@ -350,12 +407,29 @@ write_audio_frame (GeglProperties *o, AVFormatContext * oc, AVStream * st)
     p->audio_pos += sample_count;
     p->audio_track = g_list_append (p->audio_track, af);
   }
-
+  else
+  {
+    int i;
+    GeglAudioFragment *af;
+    sample_count = samples_per_frame (p->frame_count, o->frame_rate, o->audio_sample_rate, NULL);
+    af = gegl_audio_fragment_new (sample_count,
+                                                     2,
+                                                     0,
+                                                     sample_count);
+    gegl_audio_fragment_set_sample_count (af, sample_count);
+    gegl_audio_fragment_set_pos (af, p->audio_pos);
+    for (i = 0; i < sample_count; i++)
+      {
+        af->data[0][i] = 0.0;
+        af->data[1][i] = 0.0;
+      }
+    p->audio_pos += sample_count;
+    p->audio_track = g_list_append (p->audio_track, af);
+  }
   if (!(c->codec->capabilities & AV_CODEC_CAP_VARIABLE_FRAME_SIZE))
     sample_count = c->frame_size;
 
   /* then we encode as much as we can in a loop using the codec frame size */
-
   
   while (p->audio_pos - p->audio_read_pos > sample_count)
   {
@@ -439,7 +513,7 @@ write_audio_frame (GeglProperties *o, AVFormatContext * oc, AVStream * st)
     {
       pkt.stream_index = st->index;
       av_interleaved_write_frame (oc, &pkt);
-      av_free_packet (&pkt);
+      av_packet_unref (&pkt);
     }
 
     av_frame_free (&frame);
@@ -474,53 +548,73 @@ add_video_stream (GeglProperties *o, AVFormatContext * oc, int codec_id)
   c->codec_id = codec_id;
   c->codec_type = AVMEDIA_TYPE_VIDEO;
   /* put sample propeters */
-  c->bit_rate = 810000;
+  c->bit_rate = o->video_bit_rate * 1000;
+#ifdef USE_FINE_GRAINED_FFMPEG
+  c->rc_min_rate = o->video_bit_rate_min * 1000;
+  c->rc_max_rate = o->video_bit_rate_max * 1000;
+  if (o->video_bit_rate_tolerance >= 0)
+    c->bit_rate_tolerance = o->video_bit_rate_tolerance * 1000;
+#endif
   /* resolution must be a multiple of two */
   c->width = p->width;
   c->height = p->height;
   /* frames per second */
-  st->time_base =(AVRational){1, o->frame_rate};
+  st->time_base =(AVRational){1000, o->frame_rate * 1000};
   c->time_base = st->time_base;
+
   c->pix_fmt = AV_PIX_FMT_YUV420P;
-  c->gop_size = 12;             /* emit one intra frame every twelve frames at most */
+  
 
   if (c->codec_id == AV_CODEC_ID_MPEG2VIDEO)
     {
       c->max_b_frames = 2;
     }
+
   if (c->codec_id == AV_CODEC_ID_H264)
    {
-#if 1
      c->qcompress = 0.6;  // qcomp=0.6
      c->me_range = 16;    // me_range=16
      c->gop_size = 250;   // g=250
-
      c->max_b_frames = 3; // bf=3
-#if 0
-     c->coder_type = 1;  // coder = 1
-     c->flags|=CODEC_FLAG_LOOP_FILTER;   // flags=+loop
-     c->me_cmp|= 1;  // cmp=+chroma, where CHROMA = 1
-     //c->partitions|=X264_PART_I8X8+X264_PART_I4X4+X264_PART_P8X8+X264_PART_B8X8; // partitions=+parti8x8+parti4x4+partp8x8+partb8x8
-     c->me_subpel_quality = 7;   // subq=7
-     c->keyint_min = 25; // keyint_min=25
-     c->scenechange_threshold = 40;  // sc_threshold=40
-     c->i_quant_factor = 0.71; // i_qfactor=0.71
-     c->b_frame_strategy = 1;  // b_strategy=1
-     c->qmin = 10;   // qmin=10
-     c->qmax = 51;   // qmax=51
-     c->max_qdiff = 4;   // qdiff=4
-     c->refs = 3;    // refs=3
-     //c->directpred = 1;  // directpred=1
-     c->trellis = 1; // trellis=1
-     //c->flags2|=AV_CODEC_FLAG2_BPYRAMID|AV_CODEC_FLAG2_MIXED_REFS|AV_CODEC_FLAG2_WPRED+CODEC_FLAG2_8X8DCT+CODEC_FLAG2_FASTPSKIP;  // flags2=+bpyramid+mixed_refs+wpred+dct8x8+fastpskip
-     //c->weighted_p_pred = 2; // wpredp=2
-
-// libx264-main.ffpreset preset
-     //c->flags2|=CODEC_FLAG2_8X8DCT;c->flags2^=CODEC_FLAG2_8X8DCT;
-#endif
-#endif
    }
 
+  if (o->video_bufsize)
+    c->rc_buffer_size = o->video_bufsize * 1000;
+#if USE_FINE_GRAINED_FFMPEG
+  if (o->global_quality)
+     c->global_quality = o->global_quality;
+  if (o->qcompress != 0.0)
+     c->qcompress = o->qcompress;
+  if (o->qblur != 0.0)
+     c->qblur = o->qblur;
+  if (o->max_qdiff != 0)
+     c->max_qdiff = o->max_qdiff;
+  if (o->me_subpel_quality != 0)
+     c->me_subpel_quality = o->me_subpel_quality;
+  if (o->i_quant_factor != 0.0)
+     c->i_quant_factor = o->i_quant_factor;
+  if (o->i_quant_offset != 0.0)
+     c->i_quant_offset = o->i_quant_offset;
+  if (o->max_b_frames)
+    c->max_b_frames = o->max_b_frames;
+  if (o->me_range)
+    c->me_range = o->me_range;
+  if (o->noise_reduction)
+    c->noise_reduction = o->noise_reduction;
+  if (o->scenechange_threshold)
+    c->scenechange_threshold = o->scenechange_threshold;
+  if (o->trellis)
+    c->trellis = o->trellis;
+  if (o->qmin)
+    c->qmin = o->qmin;
+  if (o->qmax)
+    c->qmax = o->qmax;
+  if (o->gop_size)
+    c->gop_size = o->gop_size;
+  if (o->keyint_min)
+    c->keyint_min = o->keyint_min;
+#endif
+
    if (oc->oformat->flags & AVFMT_GLOBALHEADER)
      c->flags |= CODEC_FLAG_GLOBAL_HEADER;
 
@@ -538,7 +632,7 @@ alloc_picture (int pix_fmt, int width, int height)
   picture = av_frame_alloc ();
   if (!picture)
     return NULL;
-  size = avpicture_get_size (pix_fmt, width, height + 1);
+  size = avpicture_get_size (pix_fmt, width + 1, height + 1);
   picture_buf = malloc (size);
   if (!picture_buf)
     {
@@ -550,10 +644,12 @@ alloc_picture (int pix_fmt, int width, int height)
 }
 
 static void
-open_video (Priv * p, AVFormatContext * oc, AVStream * st)
+open_video (GeglProperties *o, AVFormatContext * oc, AVStream * st)
 {
+  Priv           *p = (Priv*)o->user_data;
   AVCodec  *codec;
   AVCodecContext *c;
+  AVDictionary *codec_options = {0};
 
   c = st->codec;
 
@@ -565,8 +661,23 @@ open_video (Priv * p, AVFormatContext * oc, AVStream * st)
       exit (1);
     }
 
+  if (codec->pix_fmts){
+    int i = 0;
+    c->pix_fmt = codec->pix_fmts[0];
+    while (codec->pix_fmts[i] !=-1)
+    {
+      if (codec->pix_fmts[i] ==  AV_PIX_FMT_RGB24)
+         c->pix_fmt = AV_PIX_FMT_RGB24;
+      i++;
+    }
+  }
+#if 0
+  if (o->video_preset[0])
+    av_dict_set (&codec_options, "preset", o->video_preset, 0);
+#endif
+
   /* open the codec */
-  if (avcodec_open2 (c, codec, NULL) < 0)
+  if (avcodec_open2 (c, codec, &codec_options) < 0)
     {
       fprintf (stderr, "could not open codec\n");
       exit (1);
@@ -667,6 +778,7 @@ write_video_frame (GeglProperties *o,
          p->picture->format = c->pix_fmt;
          p->picture->width = c->width;
          p->picture->height = c->height;
+         sws_freeContext (img_convert_ctx);
         }
     }
   else
@@ -689,6 +801,7 @@ write_video_frame (GeglProperties *o,
       pkt.data = (uint8_t *) picture_ptr;
       pkt.size = sizeof (AVPicture);
       pkt.pts = picture_ptr->pts;
+      pkt.dts = picture_ptr->pts;
       av_packet_rescale_ts (&pkt, c->time_base, st->time_base);
 
       ret = av_write_frame (oc, &pkt);
@@ -696,10 +809,33 @@ write_video_frame (GeglProperties *o,
   else
     {
       /* encode the image */
-      out_size =
-        avcodec_encode_video (c,
-                              p->video_outbuf,
-                              p->video_outbuf_size, picture_ptr);
+      AVPacket pkt2;
+      int got_packet = 0;
+      av_init_packet(&pkt2);
+      pkt2.data = p->video_outbuf;
+      pkt2.size = p->video_outbuf_size;
+
+      out_size = avcodec_encode_video2(c, &pkt2, picture_ptr, &got_packet);
+    
+      if (!out_size && got_packet && c->coded_frame)
+        {
+          c->coded_frame->pts       = pkt2.pts;
+          c->coded_frame->key_frame = !!(pkt2.flags & AV_PKT_FLAG_KEY);
+          if (c->codec->capabilities & AV_CODEC_CAP_INTRA_ONLY)
+              c->coded_frame->pict_type = AV_PICTURE_TYPE_I;
+        }
+
+      if (pkt2.side_data_elems > 0)
+        {
+          int i;
+          for (i = 0; i < pkt2.side_data_elems; i++)
+            av_free(pkt2.side_data[i].data);
+          av_freep(&pkt2.side_data);
+          pkt2.side_data_elems = 0;
+        }
+
+      if (!out_size)
+        out_size = pkt2.size;
 
       /* if zero size, it means the image was buffered */
       if (out_size != 0)
@@ -712,6 +848,7 @@ write_video_frame (GeglProperties *o,
           pkt.data = p->video_outbuf;
           pkt.size = out_size;
           pkt.pts = picture_ptr->pts;
+          pkt.dts = picture_ptr->pts;
           av_packet_rescale_ts (&pkt, c->time_base, st->time_base);
           /* write the compressed frame in the media file */
           ret = av_write_frame (oc, &pkt);
@@ -734,7 +871,11 @@ tfile (GeglProperties *o)
 {
   Priv *p = (Priv*)o->user_data;
 
-  p->fmt = av_guess_format (NULL, o->path, NULL);
+  if (strcmp (o->container_format, "auto"))
+    p->fmt = av_guess_format (o->container_format, o->path, NULL);
+  else
+    p->fmt = av_guess_format (NULL, o->path, NULL);
+
   if (!p->fmt)
     {
       fprintf (stderr,
@@ -755,6 +896,39 @@ tfile (GeglProperties *o)
 
   p->video_st = NULL;
   p->audio_st = NULL;
+  
+  if (strcmp (o->video_codec, "auto"))
+  {
+    AVCodec *codec = avcodec_find_encoder_by_name (o->video_codec);
+    p->fmt->video_codec = AV_CODEC_ID_NONE;
+    if (codec)
+      p->fmt->video_codec = codec->id;
+    else
+      {
+        fprintf (stderr, "didn't find video encoder \"%s\"\navailable codecs: ", o->video_codec);
+        while ((codec = av_codec_next (codec)))
+          if (av_codec_is_encoder (codec) &&
+              avcodec_get_type (codec->id) == AVMEDIA_TYPE_VIDEO)
+          fprintf (stderr, "%s ", codec->name);
+        fprintf (stderr, "\n");
+      }
+  }
+  if (strcmp (o->audio_codec, "auto"))
+  {
+    AVCodec *codec = avcodec_find_encoder_by_name (o->audio_codec);
+    p->fmt->audio_codec = AV_CODEC_ID_NONE;
+    if (codec)
+      p->fmt->audio_codec = codec->id;
+    else
+      {
+        fprintf (stderr, "didn't find audio encoder \"%s\"\navailable codecs: ", o->audio_codec);
+        while ((codec = av_codec_next (codec)))
+          if (av_codec_is_encoder (codec) &&
+              avcodec_get_type (codec->id) == AVMEDIA_TYPE_AUDIO)
+                fprintf (stderr, "%s ", codec->name);
+        fprintf (stderr, "\n");
+      }
+  }
 
   if (p->fmt->video_codec != AV_CODEC_ID_NONE)
     {
@@ -767,7 +941,7 @@ tfile (GeglProperties *o)
 
 
   if (p->video_st)
-    open_video (p, p->oc, p->video_st);
+    open_video (o, p->oc, p->video_st);
 
   if (p->audio_st)
     open_audio (o, p->oc, p->audio_st);
@@ -780,7 +954,11 @@ tfile (GeglProperties *o)
       return -1;
     }
 
-  avformat_write_header (p->oc, NULL);
+  if (avformat_write_header (p->oc, NULL) < 0)
+  {
+    fprintf (stderr, "'%s' error writing header\n", o->path);
+    return -1;
+  }
   return 0;
 }
 
@@ -791,7 +969,7 @@ process (GeglOperation       *operation,
          gint                 level)
 {
   GeglProperties *o = GEGL_PROPERTIES (operation);
-  Priv           *p = (Priv*)o->user_data;
+  Priv *p = (Priv*)o->user_data;
   static gint     inited = 0;
 
   g_assert (input);
@@ -822,6 +1000,8 @@ static void flush_audio (GeglProperties *o)
   Priv *p = (Priv*)o->user_data;
 
   int got_packet;
+  if (!p->audio_st)
+    return;
   do
   {
     AVPacket  pkt = { 0 };
@@ -836,7 +1016,7 @@ static void flush_audio (GeglProperties *o)
         pkt.stream_index = p->audio_st->index;
         av_packet_rescale_ts (&pkt, p->audio_st->codec->time_base, p->audio_st->time_base);
         av_interleaved_write_frame (p->oc, &pkt);
-        av_free_packet (&pkt);
+        av_packet_unref (&pkt);
       }
   } while (got_packet);
 }
@@ -845,6 +1025,7 @@ static void flush_video (GeglProperties *o)
 {
   Priv *p = (Priv*)o->user_data;
   int got_packet = 0;
+  long ts = p->frame_count;
   do {
     AVPacket  pkt = { 0 };
     int ret;
@@ -857,9 +1038,11 @@ static void flush_video (GeglProperties *o)
      if (got_packet)
      {
        pkt.stream_index = p->video_st->index;
+       pkt.pts = ts;
+       pkt.dts = ts++;
        av_packet_rescale_ts (&pkt, p->video_st->codec->time_base, p->video_st->time_base);
        av_interleaved_write_frame (p->oc, &pkt);
-       av_free_packet (&pkt);
+       av_packet_unref (&pkt);
      }
   } while (got_packet);
 }
