@@ -371,8 +371,8 @@ static const char* op_c_source =
 "/*  static gboolean done=FALSE;                                               \n"
 "    if (done)                                                                 \n"
 "      return; */                                                              \n"
-"  gegl_extension_handler_register (\".jpg\", \"gegl:jpg-load\");              \n"
-"  gegl_extension_handler_register (\".jpeg\", \"gegl:jpg-load\");             \n"
+"  gegl_extension_handler_register_loader (\".jpg\", \"gegl:jpg-load\");       \n"
+"  gegl_extension_handler_register_loader (\".jpeg\", \"gegl:jpg-load\");      \n"
 "/*  done = TRUE; */                                                           \n"
 "}                                                                             \n"
 "#endif                                                                        \n"
