@@ -229,7 +229,7 @@ static const char* op_c_source =
 "  \"title\",       _(\"PNG File Saver\"),                                     \n"
 "  \"categories\" , \"output\",                                                \n"
 "  \"description\",                                                            \n"
-"        _(\"PNG image saver (passes the buffer through, saves as a side-effect.)\"),\n"
+"        _(\"PNG image saver, using libpng\"),                                 \n"
 "        NULL);                                                                \n"
 "                                                                              \n"
 "  gegl_extension_handler_register_saver (\".png\", \"gegl:png-save\");        \n"
