@@ -384,9 +384,9 @@ static const char* op_c_source =
 "    \"description\",  _(\"PPM image loader.\"),                               \n"
 "    NULL);                                                                    \n"
 "                                                                              \n"
-"  gegl_extension_handler_register (\".ppm\", \"gegl:ppm-load\");              \n"
-"  gegl_extension_handler_register (\".pgm\", \"gegl:ppm-load\");              \n"
-"  gegl_extension_handler_register (\".pnm\", \"gegl:ppm-load\");              \n"
+"  gegl_extension_handler_register_loader (\".ppm\", \"gegl:ppm-load\");       \n"
+"  gegl_extension_handler_register_loader (\".pgm\", \"gegl:ppm-load\");       \n"
+"  gegl_extension_handler_register_loader (\".pnm\", \"gegl:ppm-load\");       \n"
 "}                                                                             \n"
 "                                                                              \n"
 "#endif                                                                        \n"
