@@ -28,7 +28,6 @@ static const char* op_c_source =
 "property_file_path (path, \"File\", \"\")                                     \n"
 "  description (_(\"Path of file to load.\"))                                  \n"
 "property_int (image_num, \"Image number\", 0)                                 \n"
-"property_int (bps, \"bits per sample\", 16)                                   \n"
 "property_int (quality, \"quality\", 10)                                       \n"
 "                                                                              \n"
 "#else                                                                         \n"
@@ -38,8 +37,6 @@ static const char* op_c_source =
 "{                                                                             \n"
 "  GeglOperationSource parent_instance;                                        \n"
 "  gpointer            properties;                                             \n"
-"                                                                              \n"
-"  gchar *cached_path;  /* Path we have cached. Detects need for recache. */   \n"
 "};                                                                            \n"
 "                                                                              \n"
 "typedef struct                                                                \n"
@@ -61,9 +58,24 @@ static const char* op_c_source =
 "typedef struct {                                                              \n"
 "  libraw_data_t            *LibRaw;                                           \n"
 "  libraw_processed_image_t *image;                                            \n"
+"  gchar                    *cached_path;                                      \n"
 "} Private;                                                                    \n"
 "                                                                              \n"
-"unsigned char first_pass = 1;                                                 \n"
+"static void                                                                   \n"
+"raw_close (GeglProperties *o)                                                 \n"
+"{                                                                             \n"
+"  Private *p = (Private*)o->user_data;                                        \n"
+"  if (p->LibRaw != NULL)                                                      \n"
+"    {                                                                         \n"
+"      if (p->image != NULL)                                                   \n"
+"      {                                                                       \n"
+"        libraw_dcraw_clear_mem (p->image);                                    \n"
+"        p->image = NULL;                                                      \n"
+"      }                                                                       \n"
+"      libraw_close (p->LibRaw);                                               \n"
+"      p->LibRaw = NULL;                                                       \n"
+"    }                                                                         \n"
+"}                                                                             \n"
 "                                                                              \n"
 "static void                                                                   \n"
 "prepare (GeglOperation *operation)                                            \n"
@@ -72,34 +84,45 @@ static const char* op_c_source =
 "  Private    *p = (Private*)o->user_data;                                     \n"
 "  int         ret;                                                            \n"
 "                                                                              \n"
-"  if (p == NULL && first_pass)                                                \n"
+"  if (p == NULL)                                                              \n"
 "    {                                                                         \n"
-"      first_pass = 0;                                                         \n"
-"                                                                              \n"
 "      if ((p = g_new0(Private, 1)) == NULL)                                   \n"
 "        g_warning (\"raw-load: Error creating private structure\");           \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  if (p->cached_path && !strcmp (p->cached_path, o->path))                    \n"
+"  {                                                                           \n"
+"     raw_close (o);                                                           \n"
+"  }                                                                           \n"
+"                                                                              \n"
+"  if (p->LibRaw == NULL)                                                      \n"
+"    {                                                                         \n"
+"      o->user_data = (gpointer)p;                                             \n"
+"      p->LibRaw = NULL;                                                       \n"
+"      p->image = NULL;                                                        \n"
+"                                                                              \n"
+"      if ((p->LibRaw = libraw_init(0)) == NULL)                               \n"
+"        g_warning (\"raw-load: Error Initializing raw library\");             \n"
 "      else                                                                    \n"
 "        {                                                                     \n"
-"          o->user_data = (gpointer)p;                                         \n"
-"          p->LibRaw = NULL;                                                   \n"
-"          p->image = NULL;                                                    \n"
+"          p->LibRaw->params.shot_select = o->image_num;                       \n"
 "                                                                              \n"
-"          if ((p->LibRaw = libraw_init(0)) == NULL)                           \n"
-"            g_warning (\"raw-load: Error Initializing raw library\");         \n"
+"          p->LibRaw->params.gamm[0] = 1.0;                                    \n"
+"          p->LibRaw->params.gamm[1] = 1.0;                                    \n"
+"          p->LibRaw->params.no_auto_bright = 1;                               \n"
+"                                                                              \n"
+"          p->LibRaw->params.output_bps = 16;                                  \n"
+"          p->LibRaw->params.user_qual = o->quality;                           \n"
+"                                                                              \n"
+"          if ((ret = libraw_open_file(p->LibRaw, o->path)) != LIBRAW_SUCCESS) \n"
+"          {                                                                   \n"
+"                                                                              \n"
+"            g_warning (\"raw-load: Unable to open %s: %s\", o->path, libraw_strerror (ret));\n"
+"          }                                                                   \n"
 "          else                                                                \n"
-"            {                                                                 \n"
-"              p->LibRaw->params.shot_select = o->image_num;                   \n"
-"                                                                              \n"
-"              p->LibRaw->params.gamm[0] = 1.0;                                \n"
-"              p->LibRaw->params.gamm[1] = 1.0;                                \n"
-"              p->LibRaw->params.no_auto_bright = 1;                           \n"
-"                                                                              \n"
-"              p->LibRaw->params.output_bps = o->bps > 8 ? 16 : 8;             \n"
-"              p->LibRaw->params.user_qual = o->quality;                       \n"
-"                                                                              \n"
-"              if ((ret = libraw_open_file(p->LibRaw, o->path)) != LIBRAW_SUCCESS)\n"
-"                g_warning (\"raw-load: Unable to open %s: %s\", o->path, libraw_strerror (ret));\n"
-"            }                                                                 \n"
+"          {                                                                   \n"
+"            p->cached_path = strdup (o->path);                                \n"
+"          }                                                                   \n"
 "        }                                                                     \n"
 "    }                                                                         \n"
 "}                                                                             \n"
@@ -141,11 +164,7 @@ static const char* op_c_source =
 "  const Babl *format = NULL;                                                  \n"
 "  int ret;                                                                    \n"
 "                                                                              \n"
-"  if (p == NULL)                                                              \n"
-"    {                                                                         \n"
-"      prepare(operation);                                                     \n"
-"      p = (Private*)o->user_data;                                             \n"
-"    }                                                                         \n"
+"  g_assert (p);                                                               \n"
 "                                                                              \n"
 "  if (p != NULL &&                                                            \n"
 "      p->LibRaw != NULL)                                                      \n"
@@ -171,20 +190,10 @@ static const char* op_c_source =
 "      rect.width  = p->image->width;                                          \n"
 "      rect.height = p->image->height;                                         \n"
 "                                                                              \n"
-"      if (p->image->bits == 8)                                                \n"
-"        {                                                                     \n"
-"          if (p->image->colors == 1)                                          \n"
-"            format = babl_format (\"Y u8\");                                  \n"
-"          else // 3 color channels                                            \n"
-"            format = babl_format (\"RGB u8\");                                \n"
-"        }                                                                     \n"
-"      else // 16-bit                                                          \n"
-"        {                                                                     \n"
-"          if (p->image->colors == 1)                                          \n"
-"            format = babl_format (\"Y u16\");                                 \n"
-"          else // 3 color channels                                            \n"
-"            format = babl_format (\"RGB u16\");                               \n"
-"        }                                                                     \n"
+"      if (p->image->colors == 1)                                              \n"
+"        format = babl_format (\"Y u16\");                                     \n"
+"      else // 3 color channels                                                \n"
+"        format = babl_format (\"RGB u16\");                                   \n"
 "                                                                              \n"
 "      gegl_buffer_set (output, &rect, 0, format, p->image->data, GEGL_AUTO_ROWSTRIDE);\n"
 "      return TRUE;                                                            \n"
@@ -200,15 +209,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "  if (o->user_data)                                                           \n"
 "    {                                                                         \n"
-"      Private *p = (Private*)o->user_data;                                    \n"
-"      if (p->LibRaw != NULL)                                                  \n"
-"        {                                                                     \n"
-"          if (p->image != NULL)                                               \n"
-"            libraw_dcraw_clear_mem (p->image);                                \n"
-"                                                                              \n"
-"          libraw_close (p->LibRaw);                                           \n"
-"        }                                                                     \n"
-"                                                                              \n"
+"      raw_close (o);                                                          \n"
 "      g_free (o->user_data);                                                  \n"
 "      o->user_data = NULL;                                                    \n"
 "    }                                                                         \n"
@@ -245,14 +246,14 @@ static const char* op_c_source =
 "    return;                                                                   \n"
 "                                                                              \n"
 "  /* query libopenraw instead. need a new API */                              \n"
-"  gegl_extension_handler_register (\".pef\", \"gegl:raw-load\");              \n"
-"  gegl_extension_handler_register (\".nef\", \"gegl:raw-load\");              \n"
-"  gegl_extension_handler_register (\".raf\", \"gegl:raw-load\");              \n"
-"  gegl_extension_handler_register (\".orf\", \"gegl:raw-load\");              \n"
-"  gegl_extension_handler_register (\".erf\", \"gegl:raw-load\");              \n"
-"  gegl_extension_handler_register (\".mrw\", \"gegl:raw-load\");              \n"
-"  gegl_extension_handler_register (\".crw\", \"gegl:raw-load\");              \n"
-"  gegl_extension_handler_register (\".cr2\", \"gegl:raw-load\");              \n"
+"  gegl_extension_handler_register_loader (\".pef\", \"gegl:raw-load\");       \n"
+"  gegl_extension_handler_register_loader (\".nef\", \"gegl:raw-load\");       \n"
+"  gegl_extension_handler_register_loader (\".raf\", \"gegl:raw-load\");       \n"
+"  gegl_extension_handler_register_loader (\".orf\", \"gegl:raw-load\");       \n"
+"  gegl_extension_handler_register_loader (\".erf\", \"gegl:raw-load\");       \n"
+"  gegl_extension_handler_register_loader (\".mrw\", \"gegl:raw-load\");       \n"
+"  gegl_extension_handler_register_loader (\".crw\", \"gegl:raw-load\");       \n"
+"  gegl_extension_handler_register_loader (\".cr2\", \"gegl:raw-load\");       \n"
 "                                                                              \n"
 "  done = TRUE;                                                                \n"
 "}                                                                             \n"
