@@ -24,72 +24,103 @@ static const char* op_c_source =
 "#ifdef GEGL_PROPERTIES                                                        \n"
 "                                                                              \n"
 "property_file_path (path, _(\"File\"), \"\")                                  \n"
-"    description    (_(\"Path to SVG file to load\"))                          \n"
-"                                                                              \n"
-"property_int (width,  _(\"Width\"),  -1)                                      \n"
-"    description (_(\"Width for rendered image\"))                             \n"
+"  description (_(\"Path of file to load\"))                                   \n"
+"property_uri (uri, _(\"URI\"), \"\")                                          \n"
+"  description (_(\"URI for file to load\"))                                   \n"
 "                                                                              \n"
+"property_int (width, _(\"Width\"), -1)                                        \n"
+"  description (_(\"Width for rendered image\"))                               \n"
 "property_int (height, _(\"Height\"), -1)                                      \n"
-"    description (_(\"Height for rendered image\"))                            \n"
+"  description (_(\"Height for rendered image\"))                              \n"
 "                                                                              \n"
 "#else                                                                         \n"
 "                                                                              \n"
 "#define GEGL_OP_SOURCE                                                        \n"
 "#define GEGL_OP_C_SOURCE svg-load.c                                           \n"
 "                                                                              \n"
-"#include \"gegl-op.h\"                                                        \n"
+"#include <gegl-op.h>                                                          \n"
 "#include <cairo.h>                                                            \n"
+"#include <gegl-gio-private.h>                                                 \n"
 "#include <librsvg/rsvg.h>                                                     \n"
 "                                                                              \n"
-"static void prepare (GeglOperation *operation)                                \n"
+"typedef struct                                                                \n"
 "{                                                                             \n"
-"  gegl_operation_set_format (operation, \"output\", babl_format (\"R'G'B'A u8\"));\n"
+"  GFile *file;                                                                \n"
+"                                                                              \n"
+"  RsvgHandle *handle;                                                         \n"
+"                                                                              \n"
+"  const Babl *format;                                                         \n"
+"                                                                              \n"
+"  gint width;                                                                 \n"
+"  gint height;                                                                \n"
+"} Priv;                                                                       \n"
+"                                                                              \n"
+"static void                                                                   \n"
+"cleanup (GeglOperation *operation)                                            \n"
+"{                                                                             \n"
+"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
+"  Priv *p = (Priv*) o->user_data;                                             \n"
+"                                                                              \n"
+"  if (p != NULL)                                                              \n"
+"    {                                                                         \n"
+"      if (p->handle != NULL)                                                  \n"
+"        g_clear_object (&p->handle);                                          \n"
+"                                                                              \n"
+"      if (p->file != NULL)                                                    \n"
+"        g_clear_object(&p->file);                                             \n"
+"                                                                              \n"
+"      p->width = p->height = 0;                                               \n"
+"      p->format = NULL;                                                       \n"
+"    }                                                                         \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static gboolean                                                               \n"
+"query_svg (GeglOperation *operation)                                          \n"
+"{                                                                             \n"
+"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
+"  Priv *p = (Priv*) o->user_data;                                             \n"
+"  RsvgDimensionData dimentions;                                               \n"
+"                                                                              \n"
+"  g_return_val_if_fail (p->handle != NULL, FALSE);                            \n"
+"                                                                              \n"
+"  rsvg_handle_get_dimensions (p->handle, &dimentions);                        \n"
+"                                                                              \n"
+"  p->format = babl_format (\"R'G'B'A u8\");                                   \n"
+"                                                                              \n"
+"  p->height = dimentions.height;                                              \n"
+"  p->width = dimentions.width;                                                \n"
+"                                                                              \n"
+"  return TRUE;                                                                \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static gint                                                                   \n"
-"gegl_buffer_import_svg (GeglBuffer  *gegl_buffer,                             \n"
-"                        const gchar *path,                                    \n"
-"                        gint         width,                                   \n"
-"                        gint         height)                                  \n"
+"load_svg (GeglOperation *operation,                                           \n"
+"          GeglBuffer    *output,                                              \n"
+"          gint           width,                                               \n"
+"          gint           height)                                              \n"
 "{                                                                             \n"
+"    GeglProperties    *o = GEGL_PROPERTIES (operation);                       \n"
+"    Priv              *p = (Priv*) o->user_data;                              \n"
 "    cairo_surface_t   *surface;                                               \n"
 "    cairo_t           *cr;                                                    \n"
-"    GError            *error = NULL;                                          \n"
-"    RsvgDimensionData  svg_dimentions;                                        \n"
-"    RsvgHandle        *handle;                                                \n"
 "                                                                              \n"
-"    handle = rsvg_handle_new_from_file (path, &error);                        \n"
-"                                                                              \n"
-"    if (!handle)                                                              \n"
-"      return 1;                                                               \n"
-"                                                                              \n"
-"    rsvg_handle_get_dimensions (handle, &svg_dimentions);                     \n"
-"                                                                              \n"
-"    if (svg_dimentions.width == 0 || svg_dimentions.height == 0)              \n"
-"      return 0;                                                               \n"
-"                                                                              \n"
-"    if (width < 1)                                                            \n"
-"      width = svg_dimentions.width;                                           \n"
-"                                                                              \n"
-"    if (height < 1)                                                           \n"
-"      height = svg_dimentions.height;                                         \n"
+"    g_return_val_if_fail (p->handle != NULL, -1);                             \n"
 "                                                                              \n"
 "    surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);\n"
 "    cr = cairo_create (surface);                                              \n"
 "                                                                              \n"
-"    if (width  != svg_dimentions.width ||                                     \n"
-"        height != svg_dimentions.height)                                      \n"
+"    if (width != p->width || height != p->height)                             \n"
 "      {                                                                       \n"
 "        cairo_scale (cr,                                                      \n"
-"                     (double)width / (double)svg_dimentions.width,            \n"
-"                     (double)height / (double)svg_dimentions.height);         \n"
+"                     (double)width / (double)p->width,                        \n"
+"                     (double)height / (double)p->height);                     \n"
 "      }                                                                       \n"
 "                                                                              \n"
-"    rsvg_handle_render_cairo (handle, cr);                                    \n"
+"    rsvg_handle_render_cairo (p->handle, cr);                                 \n"
 "                                                                              \n"
 "    cairo_surface_flush (surface);                                            \n"
 "                                                                              \n"
-"    gegl_buffer_set (gegl_buffer,                                             \n"
+"    gegl_buffer_set (output,                                                  \n"
 "                     GEGL_RECTANGLE (0, 0, width, height),                    \n"
 "                     0,                                                       \n"
 "                     babl_format (\"cairo-ARGB32\"),                          \n"
@@ -98,70 +129,148 @@ static const char* op_c_source =
 "                                                                              \n"
 "    cairo_destroy (cr);                                                       \n"
 "    cairo_surface_destroy (surface);                                          \n"
-"    g_object_unref (handle);                                                  \n"
 "                                                                              \n"
 "    return 0;                                                                 \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static void                                                                   \n"
+"prepare (GeglOperation *operation)                                            \n"
+"{                                                                             \n"
+"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
+"  Priv *p = (o->user_data) ? o->user_data : g_new0 (Priv, 1);                 \n"
+"  GError *error = NULL;                                                       \n"
+"  GInputStream *stream;                                                       \n"
+"  GFile *file = NULL;                                                         \n"
+"                                                                              \n"
+"  g_assert (p != NULL);                                                       \n"
+"                                                                              \n"
+"  if (p->file != NULL && (o->uri || o->path))                                 \n"
+"    {                                                                         \n"
+"      if (o->uri && strlen (o->uri) > 0)                                      \n"
+"        file = g_file_new_for_uri (o->uri);                                   \n"
+"      else if (o->path && strlen (o->path) > 0)                               \n"
+"        file = g_file_new_for_path (o->path);                                 \n"
+"      if (file != NULL)                                                       \n"
+"        {                                                                     \n"
+"          if (!g_file_equal (p->file, file))                                  \n"
+"            cleanup (operation);                                              \n"
+"          g_object_unref (file);                                              \n"
+"        }                                                                     \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  o->user_data = (void*) p;                                                   \n"
+"                                                                              \n"
+"  if (p->handle == NULL)                                                      \n"
+"    {                                                                         \n"
+"      stream = gegl_gio_open_input_stream (o->uri, o->path, &p->file, &error);\n"
+"      if (stream == NULL)                                                     \n"
+"        {                                                                     \n"
+"          g_warning (error->message);                                         \n"
+"          g_error_free (error);                                               \n"
+"          cleanup (operation);                                                \n"
+"          return;                                                             \n"
+"        }                                                                     \n"
+"                                                                              \n"
+"      p->handle = rsvg_handle_new_from_stream_sync (stream, p->file,          \n"
+"                                                    RSVG_HANDLE_FLAGS_NONE,   \n"
+"                                                    NULL, &error);            \n"
+"      if (p->handle == NULL)                                                  \n"
+"        {                                                                     \n"
+"          g_warning (error->message);                                         \n"
+"          g_error_free (error);                                               \n"
+"          cleanup (operation);                                                \n"
+"          return;                                                             \n"
+"        }                                                                     \n"
+"                                                                              \n"
+"      if (!query_svg (operation))                                             \n"
+"        {                                                                     \n"
+"          g_warning (\"could not query SVG image file\");                     \n"
+"          cleanup (operation);                                                \n"
+"          return;                                                             \n"
+"        }                                                                     \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  gegl_operation_set_format (operation, \"output\", p->format);               \n"
+"}                                                                             \n"
+"                                                                              \n"
 "static GeglRectangle                                                          \n"
 "get_bounding_box (GeglOperation *operation)                                   \n"
 "{                                                                             \n"
-"  GeglProperties   *o      = GEGL_PROPERTIES (operation);                     \n"
-"  gint          width  = o->width;                                            \n"
-"  gint          height = o->height;                                           \n"
+"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
+"  GeglRectangle result = { 0, 0, 0, 0 };                                      \n"
+"  Priv *p = (Priv*) o->user_data;                                             \n"
+"  gint width = o->width;                                                      \n"
+"  gint height = o->height;                                                    \n"
 "                                                                              \n"
-"  if (!o->path || !strlen(o->path))                                           \n"
-"    return *GEGL_RECTANGLE(0, 0, 0, 0);                                       \n"
-"                                                                              \n"
-"  if (width < 1 || height < 1)                                                \n"
+"  if (p->handle != NULL)                                                      \n"
 "    {                                                                         \n"
-"      RsvgDimensionData  svg_dimentions;                                      \n"
-"      RsvgHandle        *handle;                                              \n"
-"      GError            *error = NULL;                                        \n"
-"                                                                              \n"
-"      handle = rsvg_handle_new_from_file (o->path, &error);                   \n"
-"                                                                              \n"
-"      if (!handle)                                                            \n"
-"        return *GEGL_RECTANGLE(0, 0, 0, 0);                                   \n"
-"                                                                              \n"
-"      rsvg_handle_get_dimensions (handle, &svg_dimentions);                   \n"
-"                                                                              \n"
 "      if (width < 1)                                                          \n"
-"        width = svg_dimentions.width;                                         \n"
-"                                                                              \n"
+"        width = p->width;                                                     \n"
 "      if (height < 1)                                                         \n"
-"        height = svg_dimentions.height;                                       \n"
+"        height = p->height;                                                   \n"
 "                                                                              \n"
-"      rsvg_handle_get_dimensions (handle, &svg_dimentions);                   \n"
+"      result.width = width;                                                   \n"
+"      result.height = height;                                                 \n"
 "    }                                                                         \n"
 "                                                                              \n"
-"  return *GEGL_RECTANGLE(0, 0, width, height);                                \n"
+"  return result;                                                              \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static gboolean                                                               \n"
 "process (GeglOperation       *operation,                                      \n"
 "         GeglBuffer          *output,                                         \n"
-"         const GeglRectangle *result_foo,                                     \n"
+"         const GeglRectangle *result,                                         \n"
 "         gint                 level)                                          \n"
 "{                                                                             \n"
 "  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
-"  gint        result;                                                         \n"
-"  gint        width, height;                                                  \n"
+"  Priv *p = (Priv*) o->user_data;                                             \n"
+"  gint width = o->width;                                                      \n"
+"  gint height = o->height;                                                    \n"
 "                                                                              \n"
-"  width  = o->width;                                                          \n"
-"  height = o->height;                                                         \n"
+"  if (p->handle != NULL)                                                      \n"
+"  {                                                                           \n"
+"    if (width < 1)                                                            \n"
+"      width = p->width;                                                       \n"
+"    if (height < 1)                                                           \n"
+"      height = p->height;                                                     \n"
 "                                                                              \n"
-"  result = gegl_buffer_import_svg (output, o->path, width, height);           \n"
-"  if (result)                                                                 \n"
-"    {                                                                         \n"
-"      g_warning (\"%s failed to open file %s for reading.\",                  \n"
-"        G_OBJECT_TYPE_NAME (operation), o->path);                             \n"
-"      return  FALSE;                                                          \n"
-"    }                                                                         \n"
+"    if (load_svg (operation, output, width, height))                          \n"
+"      {                                                                       \n"
+"        if (o->uri != NULL && strlen(o->uri) > 0)                             \n"
+"          g_warning (\"failed to render SVG from %s\", o->uri);               \n"
+"        else                                                                  \n"
+"          g_warning (\"failed to render SVG from %s\", o->path);              \n"
+"        return FALSE;                                                         \n"
+"      }                                                                       \n"
 "                                                                              \n"
-"  return  TRUE;                                                               \n"
+"    return TRUE;                                                              \n"
+"  }                                                                           \n"
+"                                                                              \n"
+"  return FALSE;                                                               \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static GeglRectangle                                                          \n"
+"get_cached_region (GeglOperation       *operation,                            \n"
+"                   const GeglRectangle *roi)                                  \n"
+"{                                                                             \n"
+"  return get_bounding_box (operation);                                        \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static void                                                                   \n"
+"finalize (GObject *object)                                                    \n"
+"{                                                                             \n"
+"  GeglProperties *o = GEGL_PROPERTIES (object);                               \n"
+"                                                                              \n"
+"  if (o->user_data != NULL)                                                   \n"
+"    {                                                                         \n"
+"      cleanup (GEGL_OPERATION (object));                                      \n"
+"      if (o->user_data != NULL)                                               \n"
+"        g_free (o->user_data);                                                \n"
+"      o->user_data = NULL;                                                    \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  G_OBJECT_CLASS (gegl_op_parent_class)->finalize (object);                   \n"
+"}                                                                             \n"
 "                                                                              \n"
 "static void                                                                   \n"
 "gegl_op_class_init (GeglOpClass *klass)                                       \n"
@@ -169,12 +278,15 @@ static const char* op_c_source =
 "  GeglOperationClass       *operation_class;                                  \n"
 "  GeglOperationSourceClass *source_class;                                     \n"
 "                                                                              \n"
+"  G_OBJECT_CLASS(klass)->finalize = finalize;                                 \n"
+"                                                                              \n"
 "  operation_class = GEGL_OPERATION_CLASS (klass);                             \n"
 "  source_class    = GEGL_OPERATION_SOURCE_CLASS (klass);                      \n"
 "                                                                              \n"
 "  source_class->process = process;                                            \n"
 "  operation_class->prepare = prepare;                                         \n"
 "  operation_class->get_bounding_box = get_bounding_box;                       \n"
+"  operation_class->get_cached_region = get_cached_region;                     \n"
 "                                                                              \n"
 "  gegl_operation_class_set_keys (operation_class,                             \n"
 "    \"name\",         \"gegl:svg-load\",                                      \n"
@@ -183,8 +295,8 @@ static const char* op_c_source =
 "    \"description\" , _(\"Load an SVG file using librsvg\"),                  \n"
 "    NULL);                                                                    \n"
 "                                                                              \n"
-"  gegl_extension_handler_register (\".svg\", \"gegl:svg-load\");              \n"
-"  gegl_extension_handler_register (\".svgz\", \"gegl:svg-load\");             \n"
+"  gegl_extension_handler_register_loader (\".svg\", \"gegl:svg-load\");       \n"
+"  gegl_extension_handler_register_loader (\".svgz\", \"gegl:svg-load\");      \n"
 "}                                                                             \n"
 "                                                                              \n"
 "#endif                                                                        \n"
