@@ -39,6 +39,7 @@ property_int(directory, _("Directory"), 1)
 
 #include <gegl-op.h>
 #include <gegl-gio-private.h>
+#include <glib/gprintf.h>
 #include <tiffio.h>
 
 typedef enum {
@@ -105,11 +106,37 @@ lseek_to_seek_type(int whence)
     case SEEK_CUR:
       return G_SEEK_CUR;
 
-    case G_SEEK_END:
-      return SEEK_END;
+    case SEEK_END:
+      return G_SEEK_END;
     }
 }
 
+static void
+error_handler(const char *module,
+              const char *format,
+              va_list arguments)
+{
+  gchar *message;
+
+  g_vasprintf(&message, format, arguments);
+  g_warning(message);
+
+  g_free(message);
+}
+
+static void
+warning_handler(const char *module,
+                const char *format,
+                va_list arguments)
+{
+  gchar *message;
+
+  g_vasprintf(&message, format, arguments);
+  g_message(message);
+
+  g_free(message);
+}
+
 static tsize_t
 read_from_stream(thandle_t handle,
                  tdata_t buffer,
@@ -505,7 +532,7 @@ load_RGBA(GeglOperation *operation,
       GeglRectangle line = { 0, p->height - row - 1, p->width, 1 };
 #if G_BYTE_ORDER != G_LITTLE_ENDIAN
       guint row_start = row * p->width;
-      guint row end = row * p->width + p->width;
+      guint row_end = row * p->width + p->width;
       guint i;
 
       for (i = row_start; i < row_end; i++)
@@ -707,16 +734,19 @@ prepare(GeglOperation *operation)
           return;
         }
 
+      TIFFSetErrorHandler(error_handler);
+      TIFFSetWarningHandler(warning_handler);
+
       p->tiff = TIFFClientOpen("GEGL-tiff-load", "r", (thandle_t) p,
                                read_from_stream, write_to_stream,
                                seek_in_stream, close_stream,
                                get_file_size, NULL, NULL);
       if (p->tiff == NULL)
         {
-          g_warning("failed to open TIFF from %s", o->path);
-          g_input_stream_close(p->stream, NULL, NULL);
-          if (p->file != NULL)
-            g_object_unref(p->file);
+          if (o->uri != NULL && strlen(o->uri) > 0)
+            g_warning("failed to open TIFF from %s", o->uri);
+          else
+            g_warning("failed to open TIFF from %s", o->path);
           cleanup(operation);
           return;
         }
@@ -839,8 +869,8 @@ gegl_op_class_init(GeglOpClass *klass)
     "description", _("TIFF image loader using libtiff"),
     NULL);
 
-  gegl_extension_handler_register(".tiff", "gegl:tiff-load");
-  gegl_extension_handler_register(".tif", "gegl:tiff-load");
+  gegl_extension_handler_register_loader(".tiff", "gegl:tiff-load");
+  gegl_extension_handler_register_loader(".tif", "gegl:tiff-load");
 }
 
 #endif
