@@ -40,6 +40,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "#include <gegl-op.h>                                                          \n"
 "#include <gegl-gio-private.h>                                                 \n"
+"#include <glib/gprintf.h>                                                     \n"
 "#include <tiffio.h>                                                           \n"
 "                                                                              \n"
 "typedef enum {                                                                \n"
@@ -106,11 +107,37 @@ static const char* op_c_source =
 "    case SEEK_CUR:                                                            \n"
 "      return G_SEEK_CUR;                                                      \n"
 "                                                                              \n"
-"    case G_SEEK_END:                                                          \n"
-"      return SEEK_END;                                                        \n"
+"    case SEEK_END:                                                            \n"
+"      return G_SEEK_END;                                                      \n"
 "    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static void                                                                   \n"
+"error_handler(const char *module,                                             \n"
+"              const char *format,                                             \n"
+"              va_list arguments)                                              \n"
+"{                                                                             \n"
+"  gchar *message;                                                             \n"
+"                                                                              \n"
+"  g_vasprintf(&message, format, arguments);                                   \n"
+"  g_warning(message);                                                         \n"
+"                                                                              \n"
+"  g_free(message);                                                            \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static void                                                                   \n"
+"warning_handler(const char *module,                                           \n"
+"                const char *format,                                           \n"
+"                va_list arguments)                                            \n"
+"{                                                                             \n"
+"  gchar *message;                                                             \n"
+"                                                                              \n"
+"  g_vasprintf(&message, format, arguments);                                   \n"
+"  g_message(message);                                                         \n"
+"                                                                              \n"
+"  g_free(message);                                                            \n"
+"}                                                                             \n"
+"                                                                              \n"
 "static tsize_t                                                                \n"
 "read_from_stream(thandle_t handle,                                            \n"
 "                 tdata_t buffer,                                              \n"
@@ -506,7 +533,7 @@ static const char* op_c_source =
 "      GeglRectangle line = { 0, p->height - row - 1, p->width, 1 };           \n"
 "#if G_BYTE_ORDER != G_LITTLE_ENDIAN                                           \n"
 "      guint row_start = row * p->width;                                       \n"
-"      guint row end = row * p->width + p->width;                              \n"
+"      guint row_end = row * p->width + p->width;                              \n"
 "      guint i;                                                                \n"
 "                                                                              \n"
 "      for (i = row_start; i < row_end; i++)                                   \n"
@@ -708,16 +735,19 @@ static const char* op_c_source =
 "          return;                                                             \n"
 "        }                                                                     \n"
 "                                                                              \n"
+"      TIFFSetErrorHandler(error_handler);                                     \n"
+"      TIFFSetWarningHandler(warning_handler);                                 \n"
+"                                                                              \n"
 "      p->tiff = TIFFClientOpen(\"GEGL-tiff-load\", \"r\", (thandle_t) p,      \n"
 "                               read_from_stream, write_to_stream,             \n"
 "                               seek_in_stream, close_stream,                  \n"
 "                               get_file_size, NULL, NULL);                    \n"
 "      if (p->tiff == NULL)                                                    \n"
 "        {                                                                     \n"
-"          g_warning(\"failed to open TIFF from %s\", o->path);                \n"
-"          g_input_stream_close(p->stream, NULL, NULL);                        \n"
-"          if (p->file != NULL)                                                \n"
-"            g_object_unref(p->file);                                          \n"
+"          if (o->uri != NULL && strlen(o->uri) > 0)                           \n"
+"            g_warning(\"failed to open TIFF from %s\", o->uri);               \n"
+"          else                                                                \n"
+"            g_warning(\"failed to open TIFF from %s\", o->path);              \n"
 "          cleanup(operation);                                                 \n"
 "          return;                                                             \n"
 "        }                                                                     \n"
@@ -840,8 +870,8 @@ static const char* op_c_source =
 "    \"description\", _(\"TIFF image loader using libtiff\"),                  \n"
 "    NULL);                                                                    \n"
 "                                                                              \n"
-"  gegl_extension_handler_register(\".tiff\", \"gegl:tiff-load\");             \n"
-"  gegl_extension_handler_register(\".tif\", \"gegl:tiff-load\");              \n"
+"  gegl_extension_handler_register_loader(\".tiff\", \"gegl:tiff-load\");      \n"
+"  gegl_extension_handler_register_loader(\".tif\", \"gegl:tiff-load\");       \n"
 "}                                                                             \n"
 "                                                                              \n"
 "#endif                                                                        \n"
