@@ -32,6 +32,7 @@ property_file_path (path, _("File"), "")
 
 #include <gegl-op.h>
 #include <gegl-gio-private.h>
+#include <glib/gprintf.h>
 #include <tiffio.h>
 
 typedef struct
@@ -80,11 +81,37 @@ lseek_to_seek_type(int whence)
     case SEEK_CUR:
       return G_SEEK_CUR;
 
-    case G_SEEK_END:
-      return SEEK_END;
+    case SEEK_END:
+      return G_SEEK_END;
     }
 }
 
+static void
+error_handler(const char *module,
+              const char *format,
+              va_list arguments)
+{
+  gchar *message;
+
+  g_vasprintf(&message, format, arguments);
+  g_warning(message);
+
+  g_free(message);
+}
+
+static void
+warning_handler(const char *module,
+                const char *format,
+                va_list arguments)
+{
+  gchar *message;
+
+  g_vasprintf(&message, format, arguments);
+  g_message(message);
+
+  g_free(message);
+}
+
 static tsize_t
 read_from_stream(thandle_t handle,
                  tdata_t buffer,
@@ -544,6 +571,9 @@ process(GeglOperation *operation,
       goto cleanup;
     }
 
+  TIFFSetErrorHandler(error_handler);
+  TIFFSetWarningHandler(warning_handler);
+
   p->tiff = TIFFClientOpen("GEGL-tiff-save", "w", (thandle_t) p,
                            read_from_stream, write_to_stream,
                            seek_in_stream, close_stream,
@@ -551,7 +581,7 @@ process(GeglOperation *operation,
   if (p->tiff == NULL)
     {
       status = FALSE;
-      g_warning("failed to openi TIFF from %s", o->path);
+      g_warning("failed to open TIFF from %s", o->path);
       goto cleanup;
     }
 
