@@ -33,6 +33,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "#include <gegl-op.h>                                                          \n"
 "#include <gegl-gio-private.h>                                                 \n"
+"#include <glib/gprintf.h>                                                     \n"
 "#include <tiffio.h>                                                           \n"
 "                                                                              \n"
 "typedef struct                                                                \n"
@@ -81,11 +82,37 @@ static const char* op_c_source =
 "    case SEEK_CUR:                                                            \n"
 "      return G_SEEK_CUR;                                                      \n"
 "                                                                              \n"
-"    case G_SEEK_END:                                                          \n"
-"      return SEEK_END;                                                        \n"
+"    case SEEK_END:                                                            \n"
+"      return G_SEEK_END;                                                      \n"
 "    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static void                                                                   \n"
+"error_handler(const char *module,                                             \n"
+"              const char *format,                                             \n"
+"              va_list arguments)                                              \n"
+"{                                                                             \n"
+"  gchar *message;                                                             \n"
+"                                                                              \n"
+"  g_vasprintf(&message, format, arguments);                                   \n"
+"  g_warning(message);                                                         \n"
+"                                                                              \n"
+"  g_free(message);                                                            \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static void                                                                   \n"
+"warning_handler(const char *module,                                           \n"
+"                const char *format,                                           \n"
+"                va_list arguments)                                            \n"
+"{                                                                             \n"
+"  gchar *message;                                                             \n"
+"                                                                              \n"
+"  g_vasprintf(&message, format, arguments);                                   \n"
+"  g_message(message);                                                         \n"
+"                                                                              \n"
+"  g_free(message);                                                            \n"
+"}                                                                             \n"
+"                                                                              \n"
 "static tsize_t                                                                \n"
 "read_from_stream(thandle_t handle,                                            \n"
 "                 tdata_t buffer,                                              \n"
@@ -545,6 +572,9 @@ static const char* op_c_source =
 "      goto cleanup;                                                           \n"
 "    }                                                                         \n"
 "                                                                              \n"
+"  TIFFSetErrorHandler(error_handler);                                         \n"
+"  TIFFSetWarningHandler(warning_handler);                                     \n"
+"                                                                              \n"
 "  p->tiff = TIFFClientOpen(\"GEGL-tiff-save\", \"w\", (thandle_t) p,          \n"
 "                           read_from_stream, write_to_stream,                 \n"
 "                           seek_in_stream, close_stream,                      \n"
@@ -552,7 +582,7 @@ static const char* op_c_source =
 "  if (p->tiff == NULL)                                                        \n"
 "    {                                                                         \n"
 "      status = FALSE;                                                         \n"
-"      g_warning(\"failed to openi TIFF from %s\", o->path);                   \n"
+"      g_warning(\"failed to open TIFF from %s\", o->path);                    \n"
 "      goto cleanup;                                                           \n"
 "    }                                                                         \n"
 "                                                                              \n"
